# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('stories', '0011_story_audio_link'),
    ]

    operations = [
        migrations.AddField(
            model_name='story',
            name='written_by',
            field=models.CharField(help_text='The writer of the story', max_length=255, null=True, blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations

def add_category_to_current_stories(apps, schema_editor):
    Story = apps.get_model('stories', "Story")
    Category = apps.get_model('categories', "Category")

    for story in Story.objects.all():
        general_category = Category.objects.get(name__icontains="general", org=story.org)
        story.category = general_category
        story.save()


class Migration(migrations.Migration):

    dependencies = [
        ('stories', '0005_story_category'),
    ]

    operations = [
        migrations.RunPython(add_category_to_current_stories),
    ]



from __future__ import unicode_literals

from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.template import loader, Context


def send_dash_email(to_email, subject, template, context):
    from_email = getattr(settings, 'DEFAULT_FROM_EMAIL', 'website@textit.in')

    html_template = loader.get_template(template + ".html")
    text_template = loader.get_template(template + ".txt")

    context['subject'] = subject

    html = html_template.render(Context(context))
    text = text_template.render(Context(context))

    message = EmailMultiAlternatives(subject, text, from_email, [to_email])
    message.attach_alternative(html, "text/html")
    message.send()



from __future__ import unicode_literals

import re

from django import forms
from django.conf import settings
from django.contrib import messages
from django.contrib.auth import authenticate, login
from django.contrib.auth.models import User
from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse
from django.core.validators import validate_email
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _
from smartmin.views import (
    SmartCRUDL, SmartCreateView, SmartReadView, SmartUpdateView,
    SmartListView, SmartFormView, SmartTemplateView)
from .forms import CreateOrgLoginForm, OrgForm
from .models import Org, OrgBackground, Invitation, TaskState


class OrgPermsMixin(object):
    """
    Get the organisation and the user within the inheriting view so that it be
    come easy to decide whether this user has a certain permission for that
    particular organization to perform the view's actions
    """
    def get_user(self):
        return self.request.user

    def derive_org(self):
        return self.request.org

    def pre_process(self, request, *args, **kwargs):
        user = self.get_user()
        org = self.derive_org()

        if user.is_superuser:
            return None

        if not org:
            return HttpResponseRedirect(reverse('orgs.org_choose'))

        return None

    def has_org_perm(self, permission):
        (app_label, codename) = permission.split(".")

        if self.get_user().is_superuser:
            return True

        if self.get_user().is_anonymous():
            return False

        if self.org:
            org_group = self.get_user().get_org_group()
            if org_group:
                if org_group.permissions.filter(content_type__app_label=app_label,
                                                codename=codename).exists():
                    return True

        return False

    def has_permission(self, request, *args, **kwargs):
        """
        Figures out if the current user has permissions for this view.
        """
        self.kwargs = kwargs
        self.args = args
        self.request = request
        self.org = self.derive_org()

        if self.get_user().is_superuser:
            return True

        if self.get_user().has_perm(self.permission):
            return True

        return self.has_org_perm(self.permission)


class OrgObjPermsMixin(OrgPermsMixin):

    def get_object_org(self):
        return self.get_object().org

    def has_org_perm(self, codename):
        has_org_perm = super(OrgObjPermsMixin, self).has_org_perm(codename)

        if has_org_perm:
            user = self.get_user()

            if user.is_anonymous():
                return True
            return user.get_org() == self.get_object_org()

        return False

    def has_permission(self, request, *args, **kwargs):
        has_perm = super(OrgObjPermsMixin, self).has_permission(request, *args, **kwargs)

        if has_perm:
            user = self.get_user()

            # user has global permission
            if user.has_perm(self.permission):
                return True

            return user.get_org() == self.get_object_org()

        return False


class InferOrgMixin(object):
    @classmethod
    def derive_url_pattern(cls, path, action):
        return r'^%s/%s/$' % (path, action)

    def get_object(self, *args, **kwargs):
        return self.request.org


class OrgCRUDL(SmartCRUDL):
    actions = ('create', 'list', 'update', 'choose', 'home', 'edit',
               'manage_accounts', 'create_login', 'join', 'chooser')
    model = Org

    class Chooser(SmartTemplateView):
        permission = False
        template_name = getattr(settings, 'SITE_CHOOSER_TEMPLATE', 'orgs/org_chooser.html')

        def get_context_data(self, **kwargs):
            all_orgs = Org.objects.filter(is_active=True).order_by('name')

            # populate a 'host' attribute on each org so we can link off to them
            for org in all_orgs:
                org.host = org.build_host_link()

            return dict(orgs=all_orgs)

    class Create(SmartCreateView):
        form_class = OrgForm
        fields = ('name', 'language', 'timezone', 'subdomain',
                  'domain', 'api_token', 'logo', 'administrators')

    class Update(SmartUpdateView):
        form_class = OrgForm
        fields = ('is_active', 'name', 'language', 'timezone', 'subdomain',
                  'domain', 'api_token', 'logo', 'administrators')

    class List(SmartListView):
        fields = ('name', 'timezone', 'created_on', 'modified_on')

    class Choose(SmartFormView):
        class ChooseForm(forms.Form):
            def __init__(self, *args, **kwargs):
                self.user = kwargs['user']
                del kwargs['user']

                super(OrgCRUDL.Choose.ChooseForm, self).__init__(*args, **kwargs)
                self.fields['organization'].queryset = self.user.get_user_orgs()

            organization = forms.ModelChoiceField(queryset=Org.objects.filter(id__lte=-1),
                                                  empty_label=None)

        form_class = ChooseForm
        fields = ('organization',)
        title = _("Select your Organization")

        def pre_process(self, request, *args, **kwargs):
            if self.request.user.is_authenticated():
                user_orgs = self.request.user.get_user_orgs()

                if self.request.user.is_superuser:
                    return HttpResponseRedirect(reverse('orgs.org_list'))

                elif not user_orgs:
                    messages.info(
                        request, _("Your account is not associated to an "
                                   "organization. Please Contact the adminstrator."))
                    return HttpResponseRedirect(reverse('users.user_login'))

                elif user_orgs.count() == 1:
                    org = user_orgs[0]
                    if org and self.request.org:
                        if self.request.org == org:
                            self.request.session['org_id'] = org.pk
                            return HttpResponseRedirect(self.get_success_url())

            return None

        def get_context_data(self, **kwargs):
            context = super(OrgCRUDL.Choose, self).get_context_data(**kwargs)

            context['orgs'] = self.request.user.get_user_orgs()
            return context

        def has_permission(self, request, *args, **kwargs):
            return self.request.user.is_authenticated()

        def get_form_kwargs(self):
            kwargs = super(OrgCRUDL.Choose, self).get_form_kwargs()
            kwargs['user'] = self.request.user
            return kwargs

        def form_valid(self, form):
            org = form.cleaned_data['organization']

            if org in self.request.user.get_user_orgs():
                self.request.session['org_id'] = org.pk
                self.request.org = org

            return HttpResponseRedirect(org.build_host_link() + self.get_success_url())

        def get_success_url(self):
            return getattr(settings, 'SITE_USER_HOME', reverse('orgs.org_home'))

    class Home(InferOrgMixin, OrgPermsMixin, SmartReadView):
        title = _("Your Organization")
        fields = ('name', 'subdomain', 'api_token')

        def get_api_token(self, obj):
            if obj and obj.api_token:
                return "*" * 32 + obj.api_token[32:]
            else:
                return _("Not Set")

    class Edit(InferOrgMixin, OrgPermsMixin, SmartUpdateView):
        title = _("Your Organization")
        success_url = '@orgs.org_home'
        fields = ('name',)

        def derive_fields(self):
            fields = super(OrgCRUDL.Edit, self).derive_fields()
            is_super = self.request.user.is_superuser

            config_fields = getattr(settings, 'ORG_CONFIG_FIELDS', [])
            for config_field in config_fields:
                if is_super or not config_field.get('superuser_only', False):
                    fields.append(config_field['name'])

            return fields

        def get_form(self, form_class):
            form = super(OrgCRUDL.Edit, self).get_form(form_class)
            is_super = self.request.user.is_superuser

            # add all our configured org fields as well
            config_fields = getattr(settings, 'ORG_CONFIG_FIELDS', [])
            for config_field in config_fields:
                if is_super or not config_field.get('superuser_only', False):
                    field_name = config_field['name']
                    if field_name == 'featured_state':
                        choices = [(feature['properties']['id'], feature['properties']['name'])
                                   for feature in self.org.get_country_geojson()['features']]
                        form.fields[field_name] = forms.ChoiceField(choices=choices,
                                                                    **config_field['field'])
                    elif field_name.startswith('has_') or field_name.startswith('is_'):
                        form.fields[field_name] = forms.BooleanField(**config_field['field'])
                    else:
                        form.fields[field_name] = forms.CharField(**config_field['field'])

            return form

        def pre_save(self, obj):
            obj = super(OrgCRUDL.Edit, self).pre_save(obj)
            cleaned = self.form.cleaned_data
            is_super = self.request.user.is_superuser

            config_fields = getattr(settings, 'ORG_CONFIG_FIELDS', [])
            for config_field in config_fields:
                if is_super or not config_field.get('superuser_only', False):
                    name = config_field['name']
                    obj.set_config(name, cleaned.get(name, None))

            return obj

        def derive_initial(self):
            initial = super(OrgCRUDL.Edit, self).derive_initial()
            is_super = self.request.user.is_superuser

            config_fields = getattr(settings, 'ORG_CONFIG_FIELDS', [])
            for config_field in config_fields:
                if is_super or not config_field.get('superuser_only', False):
                    name = config_field['name']
                    initial[name] = self.object.get_config(name)

            return initial

        def get_object(self, *args, **kwargs):
            return self.request.org

    class ManageAccounts(InferOrgMixin, OrgPermsMixin, SmartUpdateView):

        class InviteForm(forms.ModelForm):
            emails = forms.CharField(label=_("Invite people to your organization"), required=False)
            user_group = forms.ChoiceField(choices=(('A', _("Administrators")),
                                                    ('E', _("Editors"))),
                                           required=True, initial='E', label=_("User group"))

            def clean_emails(self):
                emails = self.cleaned_data['emails'].lower().strip()
                if emails:
                    email_list = emails.split(',')
                    for email in email_list:
                        try:
                            validate_email(email)
                        except ValidationError:
                            raise forms.ValidationError(
                                _("One of the emails you entered is invalid."))
                return emails

            class Meta:
                model = Invitation
                fields = ('emails', 'user_group')

        form_class = InviteForm
        success_url = '@orgs.org_home'
        success_message = ""
        GROUP_LEVELS = ('administrators', 'editors')

        def derive_title(self):
            return _("Manage %(name)s Accounts") % {'name': self.get_object().name}

        def add_check_fields(self, form, objects, org_id, field_dict):
            for obj in objects:
                fields = []
                for grp_level in self.GROUP_LEVELS:
                    check_field = forms.BooleanField(required=False)
                    field_name = "%s_%d" % (grp_level, obj.id)

                    form.fields[field_name] = check_field
                    fields.append(field_name)

                field_dict[obj] = fields

        def derive_initial(self):
            self.org_users = self.get_object().get_org_users()

            initial = dict()
            for grp_level in self.GROUP_LEVELS:
                if grp_level == 'administrators':
                    assigned_users = self.get_object().get_org_admins()
                if grp_level == 'editors':
                    assigned_users = self.get_object().get_org_editors()

                for obj in assigned_users:
                    key = "%s_%d" % (grp_level, obj.id)
                    initial[key] = True

            return initial

        def get_form(self, form_class):
            form = super(OrgCRUDL.ManageAccounts, self).get_form(form_class)
            self.group_fields = dict()
            self.add_check_fields(form, self.org_users, self.get_object().pk,
                                  self.group_fields)

            return form

        def post_save(self, obj):
            obj = super(OrgCRUDL.ManageAccounts, self).post_save(obj)

            cleaned_data = self.form.cleaned_data
            user = self.request.user
            org = self.get_object()

            user_group = cleaned_data['user_group']

            emails = cleaned_data['emails'].lower().strip()
            email_list = emails.split(',')

            if emails:
                for email in email_list:

                    # if they already have an invite, update it
                    invites = Invitation.objects.filter(email=email, org=org).order_by('-pk')
                    invitation = invites.first()

                    if invitation:

                        # remove any old invites
                        invites.exclude(pk=invitation.pk).delete()

                        invitation.user_group = user_group
                        invitation.is_active = True
                        invitation.save()
                    else:
                        invitation = Invitation.objects.create(email=email,
                                                               org=org,
                                                               user_group=user_group,
                                                               created_by=user,
                                                               modified_by=user)

                    invitation.send_invitation()

            # remove all the org users
            for user in self.get_object().get_org_admins():
                if user != self.request.user:
                    self.get_object().administrators.remove(user)
                else:
                    self.get_object().administrators.add(user)
            for user in self.get_object().get_org_editors():
                self.get_object().editors.remove(user)

            # now update the org accounts
            for field in self.form.fields:
                if self.form.cleaned_data[field]:
                    matcher = re.match("(\w+)_(\d+)", field)
                    if matcher:
                        user_type = matcher.group(1)
                        user_id = matcher.group(2)
                        user = User.objects.get(pk=user_id)
                        if user_type == 'administrators':
                            self.get_object().administrators.add(user)
                        if user_type == 'editors':
                            self.get_object().editors.add(user)

            # update our org users after we've removed them
            self.org_users = self.get_object().get_org_users()

            return obj

        def get_context_data(self, **kwargs):
            context = super(OrgCRUDL.ManageAccounts, self).get_context_data(**kwargs)
            org = self.get_object()
            context['org'] = org
            context['org_users'] = self.org_users
            context['group_fields'] = self.group_fields
            context['invites'] = Invitation.objects.filter(org=org, is_active=True)

            return context

    class CreateLogin(SmartFormView):
        title = ""
        form_class = CreateOrgLoginForm
        success_message = ''
        submit_button_name = _("Create")
        permission = False

        def pre_process(self, request, *args, **kwargs):
            secret = self.kwargs.get('secret')
            org = self.get_object()
            if not org:
                messages.info(
                    request, _("Your invitation link is invalid. Please "
                               "contact your organization administrator."))
                return HttpResponseRedirect('/')

            elif request.org != org:
                redirect_path = reverse('orgs.org_create_login', args=[secret])
                redirect_path = org.build_host_link() + redirect_path
                return HttpResponseRedirect(redirect_path)

            return None

        def form_valid(self, form):
            user = Org.create_user(self.form.cleaned_data['email'],
                                   self.form.cleaned_data['password'])
            user.first_name = self.form.cleaned_data['first_name']
            user.last_name = self.form.cleaned_data['last_name']
            user.save()

            invitation = self.get_invitation()

            # log the user in
            user = authenticate(username=user.username,
                                password=self.form.cleaned_data['password'])
            login(self.request, user)

            obj = self.get_object()
            if invitation.user_group == 'A':
                obj.administrators.add(user)
            elif invitation.user_group == 'E':
                obj.editors.add(user)

            # make the invitation inactive
            invitation.is_active = False
            invitation.save()

            return super(OrgCRUDL.CreateLogin, self).form_valid(form)

        @classmethod
        def derive_url_pattern(cls, path, action):
            return r'^%s/%s/(?P<secret>\w+)/$' % (path, action)

        def get_invitation(self, **kwargs):
            invitation = None
            secret = self.kwargs.get('secret')
            invitations = Invitation.objects.filter(secret=secret, is_active=True)
            if invitations:
                invitation = invitations[0]
            return invitation

        def get_object(self, **kwargs):
            invitation = self.get_invitation()
            if invitation:
                return invitation.org

        def derive_title(self):
            org = self.get_object()
            return _("Join %(name)s") % {'name': org.name}

        def get_context_data(self, **kwargs):
            context = super(OrgCRUDL.CreateLogin, self).get_context_data(**kwargs)

            context['secret'] = self.kwargs.get('secret')
            context['org'] = self.get_object()

            return context

        def get_success_url(self):
            return getattr(settings, 'SITE_USER_HOME', reverse('orgs.org_home'))

    class Join(SmartUpdateView):
        class JoinForm(forms.ModelForm):

            class Meta:
                model = Org
                fields = ()

        success_message = ''
        form_class = JoinForm
        submit_button_name = _("Join")
        permission = False

        def pre_process(self, request, *args, **kwargs):
            secret = self.kwargs.get('secret')

            org = self.get_object()
            if not org:
                messages.info(
                    request, _("Your invitation link has expired. Please "
                               "contact your organization administrator."))
                return HttpResponseRedirect('/')
            elif request.org != org:

                redirect_path = org.build_host_link() + reverse('orgs.org_join', args=[secret])
                return HttpResponseRedirect(redirect_path)

            if not request.user.is_authenticated():
                return HttpResponseRedirect(reverse('orgs.org_create_login', args=[secret]))
            return None

        def derive_title(self):
            org = self.get_object()
            return _("Join %(name)s") % {'name': org.name}

        def save(self, org):
            org = self.get_object()
            invitation = self.get_invitation()
            if org:
                if invitation.user_group == 'A':
                    org.administrators.add(self.request.user)
                elif invitation.user_group == 'E':
                    org.editors.add(self.request.user)

                # make the invitation inactive
                invitation.is_active = False
                invitation.save()

                # set the active org on this user
                self.request.user.set_org(org)
                self.request.session['org_id'] = org.pk

        @classmethod
        def derive_url_pattern(cls, path, action):
            return r'^%s/%s/(?P<secret>\w+)/$' % (path, action)

        def get_invitation(self, **kwargs):
            invitation = None
            secret = self.kwargs.get('secret')
            invitations = Invitation.objects.filter(secret=secret, is_active=True)
            if invitations:
                invitation = invitations[0]
            return invitation

        def get_object(self, **kwargs):
            invitation = self.get_invitation()
            if invitation:
                return invitation.org

        def get_context_data(self, **kwargs):
            context = super(OrgCRUDL.Join, self).get_context_data(**kwargs)

            context['org'] = self.get_object()
            return context

        def get_success_url(self):
            return getattr(settings, 'SITE_USER_HOME', reverse('orgs.org_home'))


class OrgBackgroundCRUDL(SmartCRUDL):
    model = OrgBackground
    actions = ('create', 'update', 'list')

    class Update(OrgObjPermsMixin, SmartUpdateView):
        fields = ('is_active', 'name', 'background_type', 'image')

    class List(OrgPermsMixin, SmartListView):
        fields = ("name", "background_type")

        def derive_fields(self):
            if self.request.user.is_superuser:
                return ('org', 'name', 'background_type')
            return ('name', 'background_type')

        def get_background_type(self, obj):
            return obj.get_background_type_display()

        def get_queryset(self, **kwargs):
            queryset = super(OrgBackgroundCRUDL.List, self).get_queryset(**kwargs)

            if not self.get_user().is_superuser:
                queryset = queryset.filter(org=self.derive_org())

            return queryset

    class Create(OrgPermsMixin, SmartCreateView):

        def derive_fields(self):
            if self.request.user.is_superuser:
                return ('org', 'name', 'background_type', 'image')
            return ('name', 'background_type', 'image')

        def pre_save(self, obj):
            obj = super(OrgBackgroundCRUDL.Create, self).pre_save(obj)

            if not self.get_user().is_superuser:
                org = self.derive_org()
                if org:
                    obj.org = org

            return obj


class TaskCRUDL(SmartCRUDL):
    actions = ('list',)
    model = TaskState
    model_name = _("Task")
    path = 'task'

    class List(SmartListView):
        title = _("Tasks")
        link_fields = ('org',)
        default_order = ('org__name', 'task_key')

        def lookup_field_link(self, context, field, obj):
            if field == 'org':
                return reverse('orgs.org_update', args=[obj.org_id])
            else:
                return super(TaskCRUDL.List, self).lookup_field_link(context, field, obj)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('stories', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='story',
            name='image',
            field=models.ImageField(help_text='Any image that should be displayed with this story', null=True, upload_to='stories', blank=True),
        ),
    ]



from __future__ import unicode_literals

import phonenumbers
import pytz

from datetime import datetime
from django import template


register = template.Library()


@register.simple_tag()
def display_time(text_timestamp, org, time_format=None):

    if not time_format:
        time_format = '%b %d, %Y %H:%M'

    utc_tz = pytz.timezone('UTC')
    parsed_time = utc_tz.localize(datetime.strptime(text_timestamp, '%Y-%m-%dT%H:%M:%SZ'))
    output_time = parsed_time.astimezone(org.timezone)

    return output_time.strftime(time_format)


@register.simple_tag()
def national_phone(number_str):
    if number_str and number_str[0] == '+':
        try:
            return phonenumbers.format_number(phonenumbers.parse(number_str, None),
                                              phonenumbers.PhoneNumberFormat.NATIONAL)
        except:
            # number didn't parse, return it raw
            return number_str

    return number_str



from __future__ import unicode_literals

import inspect
import json
import logging
import six
import sys
import time

from celery import shared_task, signature
from django_redis import get_redis_connection
from django.apps import apps
from django.utils import timezone
from functools import wraps
from .models import Invitation, Org


ORG_TASK_LOCK_KEY = 'org-task-lock:%s:%s'

logger = logging.getLogger(__name__)


@shared_task(track_started=True, name='send_invitation_email_task')
def send_invitation_email_task(invitation_id):
    invitation = Invitation.objects.get(pk=invitation_id)
    invitation.send_email()


@shared_task(name='orgs.build_boundaries')
def build_boundaries():
    start = time.time()
    r = get_redis_connection()

    key = 'build_boundaries'
    if not r.get(key):
        with r.lock(key, timeout=900):
            active_orgs = Org.objects.filter(is_active=True)
            for org in active_orgs:
                logger.debug("=" * 40)
                org.build_boundaries()
    logger.debug("Task: build_boundaries took %ss" % (time.time() - start))


@shared_task(track_started=True, name='rebuild_org_boundaries')
def rebuild_org_boundaries(org_id):
    try:
        # get our org
        from .models import Org
        org = Org.objects.get(pk=org_id)
        org.build_boundaries()
    except Exception as e:
        logger.exception("Error building org boundaries refresh: %s" % str(e))


@shared_task
def trigger_org_task(task_name, queue='celery'):
    """
    Triggers the given org task to be run for all active orgs
    :param task_name: the full task name, e.g. 'myproj.myapp.tasks.do_stuff'
    :param queue: the name of the queue to send org sub-tasks to
    """
    active_orgs = apps.get_model('orgs', 'Org').objects.filter(is_active=True)
    for org in active_orgs:
        sig = signature(task_name, args=[org.pk])
        sig.apply_async(queue=queue)

    logger.info("Requested task '%s' for %d active orgs" % (task_name, len(active_orgs)))


def org_task(task_key, lock_timeout=None):
    """
    Decorator to create an org task.
    :param task_key: the task key used for state storage and locking, e.g. 'do-stuff'
    :param lock_timeout: the lock timeout in seconds
    """
    def _org_task(task_func):
        def _decorator(org_id):
            org = apps.get_model('orgs', 'Org').objects.get(pk=org_id)
            maybe_run_for_org(org, task_func, task_key, lock_timeout)

        return shared_task(wraps(task_func)(_decorator))
    return _org_task


def maybe_run_for_org(org, task_func, task_key, lock_timeout):
    """
    Runs the given task function for the specified org provided it's not already running
    :param org: the org
    :param task_func: the task function
    :param task_key: the task key
    :param lock_timeout: the lock timeout in seconds
    """
    r = get_redis_connection()
    key = ORG_TASK_LOCK_KEY % (org.pk, task_key)
    if r.get(key):
        logger.warn("Skipping for org #%d as it is still running" % org.pk)
    else:
        with r.lock(key, timeout=lock_timeout):
            state = org.get_task_state(task_key)
            if state.is_disabled:
                logger.info("Skipping for org #%d as task is marked disabled" % org.pk)
                return

            logger.info("Started for org #%d..." % org.pk)

            prev_started_on = state.last_successfully_started_on
            this_started_on = timezone.now()

            state.started_on = this_started_on
            state.ended_on = None
            state.save(update_fields=('started_on', 'ended_on'))

            num_task_args = len(inspect.getargspec(task_func).args)

            try:
                if num_task_args == 3:
                    results = task_func(org, prev_started_on, this_started_on)
                elif num_task_args == 1:
                    results = task_func(org)
                else:
                    raise ValueError("Task signature must be foo(org) or foo(org, since, until)")  # pragma: no cover

                state.ended_on = timezone.now()
                state.last_successfully_started_on = this_started_on
                state.last_results = json.dumps(results)
                state.is_failing = False
                state.save(update_fields=('ended_on', 'last_successfully_started_on', 'last_results', 'is_failing'))

                logger.info("Succeeded for org #%d with result: %s" % (org.pk, json.dumps(results)))

            except Exception:
                state.ended_on = timezone.now()
                state.last_results = None
                state.is_failing = True
                state.save(update_fields=('ended_on', 'last_results', 'is_failing'))

                six.reraise(*sys.exc_info())  # re-raise with original stack trace



from __future__ import unicode_literals

from django.conf.urls import include, url
from django.contrib import admin

urlpatterns = [
    url(r'^manage/', include('dash.orgs.urls')),
    url(r'^manage/', include('dash.stories.urls')),
    url(r'^manage/', include('dash.dashblocks.urls')),
    url(r'^manage/', include('dash.categories.urls')),
    url(r'^users/', include('dash.users.urls')),
    url(r'^admin/', include(admin.site.urls)),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('orgs', '0011_auto_20150710_1612'),
    ]

    operations = [
        migrations.AlterField(
            model_name='invitation',
            name='email',
            field=models.EmailField(help_text='The email to which we send the invitation of the viewer', max_length=254, verbose_name='Email'),
        ),
    ]



from __future__ import division, unicode_literals

import calendar
import datetime
import json
import pytz
import random
import six

from collections import OrderedDict
from dateutil.relativedelta import relativedelta
from django.core.cache import cache
from django.utils import timezone


def intersection(*args):
    """
    Return the intersection of lists, using the first list to determine item order
    """
    if not args:
        return []

    # remove duplicates from first list whilst preserving order
    base = list(OrderedDict.fromkeys(args[0]))

    if len(args) == 1:
        return base
    else:
        others = set(args[1]).intersection(*args[2:])
        return [e for e in base if e in others]


def union(*args):
    """
    Return the union of lists, ordering by first seen in any list
    """
    if not args:
        return []

    base = args[0]
    for other in args[1:]:
        base.extend(other)

    return list(OrderedDict.fromkeys(base))  # remove duplicates whilst preserving order


def random_string(length):
    """
    Generates a random alphanumeric string
    """
    # avoid things that could be mistaken ex: 'I' and '1'
    letters = "23456789ABCDEFGHJKLMNPQRSTUVWXYZ"
    return ''.join([random.choice(letters) for _ in range(length)])


def filter_dict(d, keys):
    """
    Creates a new dict from an existing dict that only has the given keys
    """
    return {k: v for k, v in six.iteritems(d) if k in keys}


def get_cacheable(cache_key, cache_ttl, calculate, recalculate=False):
    """
    Gets the result of a method call, using the given key and TTL as a cache
    """
    if not recalculate:
        cached = cache.get(cache_key)
        if cached is not None:
            return json.loads(cached)

    calculated = calculate()
    cache.set(cache_key, json.dumps(calculated), cache_ttl)

    return calculated


def get_obj_cacheable(obj, attr_name, calculate, recalculate=False):
    """
    Gets the result of a method call, using the given object and attribute name
    as a cache
    """
    if not recalculate and hasattr(obj, attr_name):
        return getattr(obj, attr_name)

    calculated = calculate()
    setattr(obj, attr_name, calculated)

    return calculated


def datetime_to_ms(dt):
    """
    Converts a datetime to a millisecond accuracy timestamp
    """
    seconds = calendar.timegm(dt.utctimetuple())
    return seconds * 1000 + int(dt.microsecond / 1000)


def ms_to_datetime(ms):
    """
    Converts a millisecond accuracy timestamp to a datetime
    """
    dt = datetime.datetime.utcfromtimestamp(ms / 1000)
    return dt.replace(microsecond=(ms % 1000) * 1000).replace(tzinfo=pytz.utc)


def get_month_range(d=None):
    """
    Gets the start (inclusive) and end (exclusive) datetimes of the current
    month in the same timezone as the given date
    """
    if not d:
        d = timezone.now()

    start = d.replace(day=1, hour=0, minute=0, second=0, microsecond=0)
    end = start + relativedelta(months=1)
    return start, end


def chunks(data, size):
    """
    Yield successive chunks from the given slice-able collection
    """
    if not isinstance(data, list):
        data = list(data)

    for i in six.moves.xrange(0, len(data), size):
        yield data[i:(i + size)]


def temba_client_flow_results_serializer(client_results):
    if not client_results:
        return client_results

    json_results = []
    for flow_result in client_results:
        flow_result_json = dict()
        flow_result_json['set'] = flow_result.set
        flow_result_json['unset'] = flow_result.unset
        flow_result_json['open_ended'] = flow_result.open_ended
        flow_result_json['label'] = flow_result.label
        flow_result_json['categories'] = [dict(label=category.label, count=category.count)
                                          for category in flow_result.categories]
        if flow_result.boundary:
            flow_result_json['boundary'] = flow_result.boundary

        json_results.append(flow_result_json)

    return json_results


def is_dict_equal(d1, d2, keys=None, ignore_none_values=True):
    """
    Compares two dictionaries to see if they are equal
    :param d1: the first dictionary
    :param d2: the second dictionary
    :param keys: the keys to limit the comparison to (optional)
    :param ignore_none_values: whether to ignore none values
    :return: true if the dictionaries are equal, else false
    """
    if keys or ignore_none_values:
        d1 = {k: v for k, v in six.iteritems(d1)
              if (keys is None or k in keys) and (v is not None or not ignore_none_values)}
        d2 = {k: v for k, v in six.iteritems(d2)
              if (keys is None or k in keys) and (v is not None or not ignore_none_values)}

    return d1 == d2



from __future__ import unicode_literals

from dash.categories.fields import CategoryChoiceField
from dash.orgs.views import OrgPermsMixin, OrgObjPermsMixin
from django import forms
from smartmin.views import SmartCRUDL, SmartCreateView, SmartListView, SmartUpdateView
from .models import Category, CategoryImage


class CategoryImageForm(forms.ModelForm):
    category = CategoryChoiceField(Category.objects.none())

    def __init__(self, *args, **kwargs):
        self.org = kwargs['org']
        del kwargs['org']

        super(CategoryImageForm, self).__init__(*args, **kwargs)
        self.fields['category'].queryset = Category.objects.filter(org=self.org).order_by('name')

    class Meta:
        model = CategoryImage
        fields = ('is_active', 'name', 'category', 'image')


class CategoryCRUDL(SmartCRUDL):
    model = Category
    actions = ('create', 'update', 'list')

    class Update(OrgObjPermsMixin, SmartUpdateView):
        fields = ('is_active', 'name')

    class List(OrgPermsMixin, SmartListView):

        def derive_fields(self):
            if self.request.user.is_superuser:
                return ('name', 'modified_on', 'created_on', 'org')
            return ('name', 'modified_on', 'created_on')

        def get_queryset(self, **kwargs):
            queryset = super(CategoryCRUDL.List, self).get_queryset(**kwargs)
            queryset = queryset.filter(org=self.derive_org())

            return queryset

    class Create(OrgPermsMixin, SmartCreateView):

        def derive_fields(self):
            if self.request.user.is_superuser:
                return ('name',  'org')
            return ('name', )

        def pre_save(self, obj):
            obj = super(CategoryCRUDL.Create, self).pre_save(obj)

            if not self.get_user().is_superuser:
                org = self.derive_org()
                if org:
                    obj.org = org

            return obj


class CategoryImageCRUDL(SmartCRUDL):
    model = CategoryImage
    actions = ('create', 'update', 'list')

    class Update(OrgObjPermsMixin, SmartUpdateView):
        form_class = CategoryImageForm
        fields = ('is_active', 'name', 'category', 'image')

        def get_object_org(self):
            return self.get_object().category.org

        def get_form_kwargs(self):
            kwargs = super(CategoryImageCRUDL.Update, self).get_form_kwargs()
            kwargs['org'] = self.request.org
            return kwargs

    class List(OrgPermsMixin, SmartListView):
        fields = ('name', 'category', 'modified_on', 'created_on')

        def get_queryset(self, **kwargs):
            queryset = super(CategoryImageCRUDL.List, self).get_queryset(**kwargs)
            queryset = queryset.filter(category__org=self.derive_org())

            return queryset

    class Create(OrgPermsMixin, SmartCreateView):
        form_class = CategoryImageForm
        fields = ('name', 'category', 'image')

        def get_form_kwargs(self):
            kwargs = super(CategoryImageCRUDL.Create, self).get_form_kwargs()
            kwargs['org'] = self.request.org
            return kwargs



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    def move_images(apps, schema_editor):
        # for each story we want to move any image currently on it and instead create a StoryImage for it
        Story = apps.get_model("stories", "Story")
        StoryImage = apps.get_model("stories", "StoryImage")

        for story in Story.objects.exclude(image=None):
            StoryImage.objects.create(story=story, image=story.image,
                                      created_by=story.created_by, modified_by=story.modified_by)

    dependencies = [
        ('stories', '0007_storyimage'),
    ]

    operations = [
        migrations.RunPython(move_images),
    ]




from __future__ import unicode_literals

import json
import itertools
import redis
import six

from dash.orgs.models import Org
from dash.utils import random_string
from django.contrib.auth.models import User
from django.http import JsonResponse
from django.test import TestCase


class DashTest(TestCase):
    """
    Base class for dashboard test cases
    """
    def setUp(self):
        super(DashTest, self).setUp()

        self.superuser = User.objects.create_superuser(username="root", email="super@user.com", password="root")

        self.clear_cache()

    @classmethod
    def clear_cache(cls):
        # we are extra paranoid here and actually hardcode redis to 'localhost' and '10'
        # Redis 10 is our testing redis db
        r = redis.StrictRedis(host='localhost', db=10)
        r.flushdb()

    def create_org(self, name, timezone, subdomain):
        return Org.objects.create(
            name=name, timezone=timezone, subdomain=subdomain, api_token=random_string(32),
            created_by=self.superuser, modified_by=self.superuser)

    def login(self, user):
        result = self.client.login(username=user.username, password=user.username)
        self.assertTrue(result, "Couldn't login as %(user)s / %(user)s" % dict(user=user.username))

    def url_get(self, subdomain, url, params=None, **extra):
        return self._url_request(subdomain, 'get', url, params, **extra)

    def url_post(self, subdomain, url, data=None, **extra):
        return self._url_request(subdomain, 'post', url, data, **extra)

    def assertLoginRedirect(self, response, subdomain, next_url):
        self.assertRedirects(response,
                             'http://%s.localhost/users/login/?next=%s' % (subdomain, next_url))

    def _url_request(self, subdomain, method, url, data, **extra):
        if data is None:
            data = {}

        if subdomain:
            extra['HTTP_HOST'] = '%s.localhost' % subdomain

        func = getattr(self.client, method)
        response = func(url, data, **extra)

        if isinstance(response, JsonResponse):
            content = response.content
            if isinstance(content, six.binary_type):
                content = content.decode('utf-8')

            response.json = json.loads(content)

        return response


class MockClientQuery(six.Iterator):
    """
    Mock for APIv2 client get_xxxxx return values. Pass lists of temba objects to mock each fetch the client would make.

    For example:
        mock_get_contacts.return_value = MockClientQuery(
            [TembaContact.create(...), TembaContact.create(...), TembaContact.create(...)]
            [TembaContact.create(...)]
        )

    Will return the three contacts on the first call to iterfetches, and one on the second call.

    """
    def __init__(self, *fetches):
        self.fetches = list(fetches)

    def iterfetches(self, *args, **kwargs):
        return self

    def all(self, *args, **kwargs):
        return list(itertools.chain.from_iterable(self.fetches))

    def first(self, *args, **kwargs):
        return self.fetches[0][0] if self.fetches[0] else None

    def get_cursor(self):
        return 'cursor-string'

    def __iter__(self):
        return self

    def __next__(self):
        if not self.fetches:
            raise StopIteration()

        return self.fetches.pop(0)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('dashblocks', '0005_auto_20140904_0957'),
    ]

    operations = [
        migrations.AlterField(
            model_name='dashblock',
            name='created_by',
            field=models.ForeignKey(related_name='dashblocks_dashblock_creations', to=settings.AUTH_USER_MODEL, help_text='The user which originally created this item'),
        ),
        migrations.AlterField(
            model_name='dashblock',
            name='modified_by',
            field=models.ForeignKey(related_name='dashblocks_dashblock_modifications', to=settings.AUTH_USER_MODEL, help_text='The user which last modified this item'),
        ),
        migrations.AlterField(
            model_name='dashblockimage',
            name='created_by',
            field=models.ForeignKey(related_name='dashblocks_dashblockimage_creations', to=settings.AUTH_USER_MODEL, help_text='The user which originally created this item'),
        ),
        migrations.AlterField(
            model_name='dashblockimage',
            name='dashblock',
            field=models.ForeignKey(related_name='images', to='dashblocks.DashBlock'),
        ),
        migrations.AlterField(
            model_name='dashblockimage',
            name='modified_by',
            field=models.ForeignKey(related_name='dashblocks_dashblockimage_modifications', to=settings.AUTH_USER_MODEL, help_text='The user which last modified this item'),
        ),
        migrations.AlterField(
            model_name='dashblocktype',
            name='created_by',
            field=models.ForeignKey(related_name='dashblocks_dashblocktype_creations', to=settings.AUTH_USER_MODEL, help_text='The user which originally created this item'),
        ),
        migrations.AlterField(
            model_name='dashblocktype',
            name='modified_by',
            field=models.ForeignKey(related_name='dashblocks_dashblocktype_modifications', to=settings.AUTH_USER_MODEL, help_text='The user which last modified this item'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('orgs', '0006_auto_20140919_2056'),
    ]

    operations = [
        migrations.AlterField(
            model_name='invitation',
            name='created_by',
            field=models.ForeignKey(related_name='orgs_invitation_creations', to=settings.AUTH_USER_MODEL, help_text='The user which originally created this item'),
        ),
        migrations.AlterField(
            model_name='invitation',
            name='modified_by',
            field=models.ForeignKey(related_name='orgs_invitation_modifications', to=settings.AUTH_USER_MODEL, help_text='The user which last modified this item'),
        ),
        migrations.AlterField(
            model_name='invitation',
            name='org',
            field=models.ForeignKey(related_name='invitations', verbose_name='Org', to='orgs.Org', help_text='The organization to which the account is invited to view'),
        ),
        migrations.AlterField(
            model_name='org',
            name='administrators',
            field=models.ManyToManyField(help_text='The administrators in your organization', related_name='org_admins', verbose_name='Administrators', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='org',
            name='created_by',
            field=models.ForeignKey(related_name='orgs_org_creations', to=settings.AUTH_USER_MODEL, help_text='The user which originally created this item'),
        ),
        migrations.AlterField(
            model_name='org',
            name='editors',
            field=models.ManyToManyField(help_text='The editors in your organization', related_name='org_editors', verbose_name='Editors', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='org',
            name='modified_by',
            field=models.ForeignKey(related_name='orgs_org_modifications', to=settings.AUTH_USER_MODEL, help_text='The user which last modified this item'),
        ),
        migrations.AlterField(
            model_name='org',
            name='viewers',
            field=models.ManyToManyField(help_text='The viewers in your organization', related_name='org_viewers', verbose_name='Viewers', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='orgbackground',
            name='created_by',
            field=models.ForeignKey(related_name='orgs_orgbackground_creations', to=settings.AUTH_USER_MODEL, help_text='The user which originally created this item'),
        ),
        migrations.AlterField(
            model_name='orgbackground',
            name='modified_by',
            field=models.ForeignKey(related_name='orgs_orgbackground_modifications', to=settings.AUTH_USER_MODEL, help_text='The user which last modified this item'),
        ),
        migrations.AlterField(
            model_name='orgbackground',
            name='org',
            field=models.ForeignKey(related_name='backgrounds', verbose_name='Org', to='orgs.Org', help_text='The organization in which the image will be used'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('orgs', '0009_auto_20150331_1452'),
    ]

    operations = [
        migrations.AlterField(
            model_name='org',
            name='subdomain',
            field=models.SlugField(null=True, error_messages={'unique': 'This subdomain is not available'}, max_length=255, blank=True, help_text='The subdomain for this U-Report instance', unique=True, verbose_name='Subdomain'),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations

def generate_initial_block_types(apps, schema_editor):
    User = apps.get_model("auth", "User")
    root = User.objects.filter(username="root").first()

    if not root:
        root = User.objects.filter(username="root2").first()

    if not root:
        root = User.objects.create(username="root2")

    DashBlockType = apps.get_model("dashblocks", "DashBlockType")

    DashBlockType.objects.get_or_create(name="Missions",
                                        slug="missions",
                                        description="U-Report missions blocks",
                                        has_title=True,
                                        has_image=True,
                                        has_rich_text=False,
                                        has_summary=False,
                                        has_link=False,
                                        has_gallery=False,
                                        has_color=False,
                                        has_video=False,
                                        has_tags=False,
                                        created_by=root,
                                        modified_by=root)

    DashBlockType.objects.get_or_create(name="Contact Us",
                                        slug="contact_us",
                                        description="U-Report contact us blocks",
                                        has_title=True,
                                        has_image=False,
                                        has_rich_text=False,
                                        has_summary=False,
                                        has_link=False,
                                        has_gallery=False,
                                        has_color=False,
                                        has_video=False,
                                        has_tags=False,
                                        created_by=root,
                                        modified_by=root)

class Migration(migrations.Migration):

    dependencies = [
        ('dashblocks', '0003_auto_20140804_0236'),
    ]

    operations = [
        migrations.RunPython(generate_initial_block_types),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.10 on 2016-10-26 15:13
from __future__ import unicode_literals

from django.db import migrations
import timezone_field.fields


class Migration(migrations.Migration):

    dependencies = [
        ('orgs', '0016_taskstate_is_disabled'),
    ]

    operations = [
        migrations.AlterField(
            model_name='org',
            name='timezone',
            field=timezone_field.fields.TimeZoneField(default='UTC', help_text='The timezone your organization is in.', verbose_name='Timezone'),
        ),
    ]



from __future__ import unicode_literals

from dash.categories.fields import CategoryChoiceField
from dash.orgs.views import OrgPermsMixin, OrgObjPermsMixin
from django import forms
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from smartmin.views import SmartCRUDL, SmartCreateView, SmartListView, SmartUpdateView
from .models import Category, Story, StoryImage


class StoryForm(forms.ModelForm):
    category = CategoryChoiceField(Category.objects.none())

    def __init__(self, *args, **kwargs):
        self.org = kwargs['org']
        del kwargs['org']
        super(StoryForm, self).__init__(*args, **kwargs)

        # We show all categories even inactive one in the dropdown
        qs = Category.objects.filter(org=self.org).order_by('name')
        self.fields['category'].queryset = qs

    class Meta:
        model = Story
        fields = ('is_active', 'title', 'featured', 'summary', 'content', 'written_by', 'audio_link',
                  'video_id', 'tags', 'category')


class StoryCRUDL(SmartCRUDL):
    model = Story
    actions = ('create', 'update', 'list', 'images')

    class Update(OrgObjPermsMixin, SmartUpdateView):
        form_class = StoryForm
        fields = (
            'is_active', 'title', 'featured', 'summary', 'content', 'written_by', 'audio_link',
            'video_id', 'tags', 'category')

        def pre_save(self, obj):
            obj = super(StoryCRUDL.Update, self).pre_save(obj)
            obj.audio_link = Story.format_audio_link(obj.audio_link)
            obj.tags = Story.space_tags(obj.tags)
            return obj

        def get_form_kwargs(self):
            kwargs = super(StoryCRUDL.Update, self).get_form_kwargs()
            kwargs['org'] = self.request.org
            return kwargs

    class List(OrgPermsMixin, SmartListView):
        fields = ('title', 'images', 'featured', 'created_on')
        search_fields = ('title__icontains',)
        link_fields = ('title', 'images',)
        default_order = ('-created_on',)

        def get_featured(self, obj):
            if obj.featured:
                return _("Yes")
            return _("No")

        def lookup_field_link(self, context, field, obj):
            if field == 'images':
                return reverse('stories.story_images', args=[obj.pk])
            else:
                return super(StoryCRUDL.List, self).lookup_field_link(context, field, obj)

        def get_images(self, obj):
            return obj.images.count()

        def get_queryset(self, **kwargs):
            queryset = super(StoryCRUDL.List, self).get_queryset(**kwargs)
            queryset = queryset.filter(org=self.derive_org())

            return queryset

    class Images(OrgObjPermsMixin, SmartUpdateView):
        success_url = '@stories.story_list'
        title = _("Story Images")

        def get_form(self, form_class):
            form = super(StoryCRUDL.Images, self).get_form(form_class)
            form.fields.clear()

            idx = 1

            # add existing images
            for image in self.object.images.all().order_by('pk'):
                image_field_name = 'image_%d' % idx
                image_field = forms.ImageField(
                    required=False, initial=image.image, label=_("Image %d") % idx,
                    help_text=_("Image to display on story page and in previews. (optional)"))

                self.form.fields[image_field_name] = image_field
                idx += 1

            while idx <= 3:
                self.form.fields['image_%d' % idx] = forms.ImageField(
                    required=False, label=_("Image %d") % idx,
                    help_text=_("Image to display on story page and in previews (optional)"))
                idx += 1

            return form

        def post_save(self, obj):
            obj = super(StoryCRUDL.Images, self).post_save(obj)

            # remove our existing images
            self.object.images.all().delete()

            # overwrite our new ones
            # TODO: this could probably be done more elegantly
            for idx in range(1, 4):
                image = self.form.cleaned_data.get('image_%d' % idx, None)

                if image:
                    StoryImage.objects.create(
                        story=self.object, image=image,
                        created_by=self.request.user, modified_by=self.request.user)

            return obj

    class Create(OrgPermsMixin, SmartCreateView):
        form_class = StoryForm
        success_url = 'id@stories.story_images'
        fields = (
            'title', 'featured', 'summary', 'content', 'written_by', 'audio_link', 'video_id', 'tags',
            'category')

        def pre_save(self, obj):
            obj = super(StoryCRUDL.Create, self).pre_save(obj)

            obj.org = self.request.org
            obj.audio_link = Story.format_audio_link(obj.audio_link)
            obj.tags = Story.space_tags(obj.tags)
            return obj

        def get_form_kwargs(self):
            kwargs = super(StoryCRUDL.Create, self).get_form_kwargs()
            kwargs['org'] = self.request.org
            return kwargs



from __future__ import unicode_literals

"""
WSGI config for dash_test_runner project.

It exposes the WSGI callable as a module-level variable named ``application``.

For more information on this file, see
https://docs.djangoproject.com/en/dev/howto/deployment/wsgi/
"""

import os
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "dash_test_runner.settings")

from django.core.wsgi import get_wsgi_application  # noqa
application = get_wsgi_application()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('orgs', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='DashBlock',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('is_active', models.BooleanField(default=True, help_text='Whether this item is active, use this instead of deleting')),
                ('created_on', models.DateTimeField(help_text='When this item was originally created', auto_now_add=True)),
                ('modified_on', models.DateTimeField(help_text='When this item was last modified', auto_now=True)),
                ('title', models.CharField(help_text='The title for this block of content, optional', max_length=255, null=True, blank=True)),
                ('summary', models.TextField(help_text='The summary for this item, should be short', null=True, blank=True)),
                ('content', models.TextField(help_text='The body of text for this content block, optional', null=True, blank=True)),
                ('image', models.ImageField(help_text='Any image that should be displayed with this content block, optional', null=True, upload_to='dashblocks', blank=True)),
                ('color', models.CharField(help_text='A background color to use for the image, in the format: #rrggbb', max_length=16, null=True, blank=True)),
                ('link', models.CharField(help_text='Any link that should be associated with this content block, optional', max_length=255, null=True, blank=True)),
                ('video_id', models.CharField(help_text='The id of the YouTube video that should be linked to this item', max_length=255, null=True, blank=True)),
                ('tags', models.CharField(help_text='Any tags for this content block, separated by spaces, can be used to do more advanced filtering, optional', max_length=255, null=True, blank=True)),
                ('priority', models.IntegerField(default=0, help_text='The priority for this block, higher priority blocks come first')),
                ('created_by', models.ForeignKey(help_text='The user which originally created this item', to=settings.AUTH_USER_MODEL)),
                ('modified_by', models.ForeignKey(help_text='The user which last modified this item', to=settings.AUTH_USER_MODEL)),
                ('org', models.ForeignKey(help_text='The organization this content block belongs to', to='orgs.Org')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='DashBlockImage',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('is_active', models.BooleanField(default=True, help_text='Whether this item is active, use this instead of deleting')),
                ('created_on', models.DateTimeField(help_text='When this item was originally created', auto_now_add=True)),
                ('modified_on', models.DateTimeField(help_text='When this item was last modified', auto_now=True)),
                ('image', models.ImageField(height_field='height', width_field='width', upload_to='dashblock_images/')),
                ('caption', models.CharField(max_length=64)),
                ('priority', models.IntegerField(default=0, null=True, blank=True)),
                ('width', models.IntegerField()),
                ('height', models.IntegerField()),
                ('created_by', models.ForeignKey(help_text='The user which originally created this item', to=settings.AUTH_USER_MODEL)),
                ('dashblock', models.ForeignKey(to='dashblocks.DashBlock')),
                ('modified_by', models.ForeignKey(help_text='The user which last modified this item', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='DashBlockType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('is_active', models.BooleanField(default=True, help_text='Whether this item is active, use this instead of deleting')),
                ('created_on', models.DateTimeField(help_text='When this item was originally created', auto_now_add=True)),
                ('modified_on', models.DateTimeField(help_text='When this item was last modified', auto_now=True)),
                ('name', models.CharField(help_text='The human readable name for this content type', unique=True, max_length=75)),
                ('slug', models.SlugField(help_text='The slug to idenfity this content type, used with the template tags', unique=True)),
                ('description', models.TextField(help_text='A description of where this content type is used on the site and how it will be dsiplayed', null=True, blank=True)),
                ('has_title', models.BooleanField(default=True, help_text='Whether this content should include a title')),
                ('has_image', models.BooleanField(default=True, help_text='Whether this content should include an image')),
                ('has_rich_text', models.BooleanField(default=True, help_text='Whether this content should use a rich HTML editor')),
                ('has_summary', models.BooleanField(default=True, help_text='Whether this content should include a summary field')),
                ('has_link', models.BooleanField(default=True, help_text='Whether this content should include a link')),
                ('has_gallery', models.BooleanField(default=False, help_text='Whether this content should allow upload of additional images, ie a gallery')),
                ('has_color', models.BooleanField(default=False, help_text='Whether this content has a color field')),
                ('has_video', models.BooleanField(default=False, help_text='Whether this content should allow setting a YouTube id')),
                ('has_tags', models.BooleanField(default=False, help_text='Whether this content should allow tags')),
                ('created_by', models.ForeignKey(help_text='The user which originally created this item', to=settings.AUTH_USER_MODEL)),
                ('modified_by', models.ForeignKey(help_text='The user which last modified this item', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='dashblock',
            name='dashblock_type',
            field=models.ForeignKey(verbose_name='Content Type', to='dashblocks.DashBlockType', help_text='The category, or type for this content block'),
            preserve_default=True,
        ),
    ]



from __future__ import unicode_literals


from dash.orgs.models import Org
from dash.utils.sync import BaseSyncer
from django.db import models
from django.utils.translation import ugettext as _
from django_redis import get_redis_connection


class Contact(models.Model):
    org = models.ForeignKey(Org)

    uuid = models.CharField(max_length=36, unique=True)

    name = models.CharField(verbose_name=_("Name"), max_length=128)

    is_active = models.BooleanField(default=True)

    @classmethod
    def lock(cls, org, uuid):
        return get_redis_connection().lock('contact-lock:%d:%s' % (org.pk, uuid), timeout=60)


class ContactSyncer(BaseSyncer):
    model = Contact

    def local_kwargs(self, org, remote):
        if remote.blocked:  # we don't store blocked contacts
            return None

        return {
            'org': org,
            'uuid': remote.uuid,
            'name': remote.name
        }

    def update_required(self, local, remote, remote_as_kwargs):
        return local.name != remote.name



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('categories', '0002_auto_20140820_1415'),
        ('stories', '0004_auto_20140806_1540'),
    ]

    operations = [
        migrations.AddField(
            model_name='story',
            name='category',
            field=models.ForeignKey(blank=True, to='categories.Category', help_text='The category for this story', null=True),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('orgs', '0015_auto_20160209_0926'),
    ]

    operations = [
        migrations.CreateModel(
            name='Contact',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('uuid', models.CharField(unique=True, max_length=36)),
                ('name', models.CharField(max_length=128, verbose_name='Name')),
                ('is_active', models.BooleanField(default=True)),
                ('org', models.ForeignKey(to='orgs.Org')),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('orgs', '0005_orgbackground'),
    ]

    operations = [
        migrations.AlterField(
            model_name='org',
            name='language',
            field=models.CharField(choices=[('en', 'English'), ('fr', 'French')], max_length=64, blank=True, help_text='The main language used by this organization', null=True, verbose_name='Language'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('categories', '0005_auto_20140922_1514'),
    ]

    operations = [
        migrations.AlterField(
            model_name='category',
            name='org',
            field=models.ForeignKey(related_name='categories', to='orgs.Org', help_text='The organization this category applies to'),
        ),
    ]



from __future__ import unicode_literals

import re
import traceback

from django.conf import settings
from django.core.exceptions import DisallowedHost
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.utils import translation, timezone
from .models import Org


ALLOW_NO_ORG = (
    'users.user_login',
    'users.user_logout',
    'users.user_create',
    'users.user_list',
    'users.user_update',
    'users.user_profile',
    'users.user_forget',
    'users.user_recover',
    'users.user_expired',
    'users.user_failed',
    'users.user_newpassword',
    'users.user_mimic',
    'orgs.org_create',
    'orgs.org_list',
    'orgs.org_update',
    'orgs.org_choose',
    'orgs.org_home',
    'orgs.org_edit',
    'orgs.org_manage_accounts',
    'orgs.org_create_login',
    'orgs.org_join',
    'orgs.orgbackground_create',
    'orgs.orgbackground_update',
    'orgs.orgbackground_list'
)


class SetOrgMiddleware(object):
    """
    Sets the org on the request, based on the subdomain
    """
    def process_request(self, request):

        # try looking the domain level
        host_parts = self.get_host_parts(request)

        org = None
        # the domain is something like 'ureport.bi' or 'ureport.co.ug'
        if len(host_parts) >= 2:
            # we might have a three part domain like 'ureport.co.ug'
            domain = ".".join(host_parts[-3:])
            org = Org.objects.filter(domain__iexact=domain, is_active=True).first()

            if not org:
                # try now the two last part for domains like 'ureport.bi'
                domain = ".".join(host_parts[-2:])
                org = Org.objects.filter(domain__iexact=domain, is_active=True).first()

        elif host_parts:
            # we have a domain like 'localhost'
            domain = host_parts[0]
            org = Org.objects.filter(domain__iexact=domain, is_active=True).first()

        # no custom domain found, try the subdomain
        if not org:
            subdomain = self.get_subdomain(request)

            org = Org.objects.filter(subdomain__iexact=subdomain, is_active=True).first()

        if not request.user.is_anonymous():
            request.user.set_org(org)

        request.org = org

        self.set_language(request, org)
        self.set_timezone(request, org)

    def set_language(self, request, org):
        """Set the current language from the org configuration."""
        if org:
            lang = org.language or settings.DEFAULT_LANGUAGE
            translation.activate(lang)

    def set_timezone(self, request, org):
        """Set the current timezone from the org configuration."""
        if org and org.timezone:
            timezone.activate(org.timezone)

    def process_view(self, request, view_func, view_args, view_kwargs):
        if not request.org:
            # serve static files
            media_url = getattr(settings, 'MEDIA_URL', None)
            static_url = getattr(settings, 'STATIC_URL', None)
            path = request.path

            if media_url and path.startswith(media_url):
                return None
            if static_url and path.startswith(static_url):
                return None

            # only some pages can be viewed without an org
            url_name = request.resolver_match.url_name
            whitelist = ALLOW_NO_ORG + getattr(settings, 'SITE_ALLOW_NO_ORG', ())

            # make sure the chooser view is whitelisted
            chooser_view = getattr(settings, 'SITE_CHOOSER_URL_NAME', 'orgs.org_chooser')
            whitelist += (chooser_view,)

            if url_name not in whitelist:
                return HttpResponseRedirect(reverse(chooser_view))

    def get_host_parts(self, request):
        host = 'localhost'
        try:
            host = request.get_host()
        except DisallowedHost:
            traceback.print_exc()

        # does the host look like an IP? return []
        if re.match("^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$", host):
            return []

        return host.split('.')

    def get_subdomain(self, request):

        subdomain = ""
        parts = self.get_host_parts(request)
        host_string = ".".join(parts)

        # we only look up subdomains for localhost and the configured hostname only
        top_domains = ['localhost:8000', 'localhost', getattr(settings, 'HOSTNAME', "")]
        allowed_top_domain = False
        for top in top_domains:
            if host_string.endswith(top):
                allowed_top_domain = True
                break

        # if empty parts or domain neither localhost nor hostname return ""
        if not parts or not allowed_top_domain:
            return subdomain

        # if we have parts for domain like 'www.nigeria.ureport.in'
        if len(parts) > 2:
            subdomain = parts[0]
            parts = parts[1:]

            # we keep stripping subdomains if the subdomain is something
            # like 'www' and there are more parts
            while subdomain.lower() == 'www' and len(parts) > 1:
                subdomain = parts[0]
                parts = parts[1:]

        elif len(parts) > 0:
            # for domains like 'ureport.in' we just take the first part
            subdomain = parts[0]

        # get the configured hostname
        hostname = getattr(settings, 'HOSTNAME', '')
        domain_first_part = hostname.lower().split('.')[0]

        # if the subdomain is the same as the first part of hostname
        # ignore than and return ''
        if subdomain.lower() in [domain_first_part, 'localhost']:
            subdomain = ""

        return subdomain



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('orgs', '0002_auto_20140802_2104'),
    ]

    operations = [
        migrations.AddField(
            model_name='org',
            name='logo',
            field=models.ImageField(help_text='The logo that should be used for this organization', null=True, upload_to='logos', blank=True),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Invitation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('is_active', models.BooleanField(default=True, help_text='Whether this item is active, use this instead of deleting')),
                ('created_on', models.DateTimeField(help_text='When this item was originally created', auto_now_add=True)),
                ('modified_on', models.DateTimeField(help_text='When this item was last modified', auto_now=True)),
                ('email', models.EmailField(help_text='The email to which we send the invitation of the viewer', max_length=75, verbose_name='Email')),
                ('secret', models.CharField(help_text='a unique code associated with this invitation', unique=True, max_length=64, verbose_name='Secret')),
                ('user_group', models.CharField(default='V', max_length=1, verbose_name='User Role', choices=[('A', 'Administrator'), ('E', 'Editor'), ('V', 'Viewer')])),
                ('created_by', models.ForeignKey(help_text='The user which originally created this item', to=settings.AUTH_USER_MODEL)),
                ('modified_by', models.ForeignKey(help_text='The user which last modified this item', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Org',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('is_active', models.BooleanField(default=True, help_text='Whether this item is active, use this instead of deleting')),
                ('created_on', models.DateTimeField(help_text='When this item was originally created', auto_now_add=True)),
                ('modified_on', models.DateTimeField(help_text='When this item was last modified', auto_now=True)),
                ('name', models.CharField(help_text='The name of this organization', max_length=128, verbose_name='Name')),
                ('language', models.CharField(choices=[('en-us', 'English'), ('rw', 'Kinyarwanda'), ('fr', 'French')], max_length=64, blank=True, help_text='The main language used by this organization', null=True, verbose_name='Language')),
                ('subdomain', models.SlugField(error_messages={'unique': 'This subdomain is not available'}, max_length=255, help_text='The subdomain for this UReport instance', unique=True, verbose_name='Subdomain')),
                ('api_token', models.CharField(help_text='The API token for the RapidPro account this dashboard is tied to', max_length=128, null=True, blank=True)),
                ('config', models.TextField(help_text='JSON blob used to store configuration information associated with this organization', null=True, blank=True)),
                ('administrators', models.ManyToManyField(to=settings.AUTH_USER_MODEL, verbose_name='Administrators')),
                ('created_by', models.ForeignKey(help_text='The user which originally created this item', to=settings.AUTH_USER_MODEL)),
                ('editors', models.ManyToManyField(to=settings.AUTH_USER_MODEL, verbose_name='Editors')),
                ('modified_by', models.ForeignKey(help_text='The user which last modified this item', to=settings.AUTH_USER_MODEL)),
                ('viewers', models.ManyToManyField(to=settings.AUTH_USER_MODEL, verbose_name='Viewers')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='invitation',
            name='org',
            field=models.ForeignKey(verbose_name='Org', to='orgs.Org', help_text='The organization to which the account is invited to view'),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


def populate_story_summary(apps, schema_editor):
    Story = apps.get_model('stories', "Story")
    for story in Story.objects.all():
        story.summary = story.content
        story.save()

class Migration(migrations.Migration):

    dependencies = [
        ('stories', '0003_story_summary'),
    ]

    operations = [
        migrations.RunPython(populate_story_summary),
    ]



from __future__ import unicode_literals

from django.conf import settings


def lang_direction(request):
    """
    Sets lang_direction context variable to whether the language is RTL or LTR
    """
    if lang_direction.rtl_langs is None:
        lang_direction.rtl_langs = getattr(settings, 'RTL_LANGUAGES', set())

    return {
        'lang_direction': 'rtl' if request.LANGUAGE_CODE in lang_direction.rtl_langs else 'ltr'
    }


lang_direction.rtl_langs = None



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('orgs', '0015_auto_20160209_0926'),
    ]

    operations = [
        migrations.AddField(
            model_name='taskstate',
            name='is_disabled',
            field=models.BooleanField(default=False),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('stories', '0009_remove_story_image'),
    ]

    operations = [
        migrations.AlterField(
            model_name='story',
            name='created_by',
            field=models.ForeignKey(related_name='stories_story_creations', to=settings.AUTH_USER_MODEL, help_text='The user which originally created this item'),
        ),
        migrations.AlterField(
            model_name='story',
            name='modified_by',
            field=models.ForeignKey(related_name='stories_story_modifications', to=settings.AUTH_USER_MODEL, help_text='The user which last modified this item'),
        ),
        migrations.AlterField(
            model_name='story',
            name='video_id',
            field=models.CharField(help_text='The id of the YouTube video that should be linked to this story (this is the text that comes afer v= and before & in the YouTube URL)', max_length=255, null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='storyimage',
            name='created_by',
            field=models.ForeignKey(related_name='stories_storyimage_creations', to=settings.AUTH_USER_MODEL, help_text='The user which originally created this item'),
        ),
        migrations.AlterField(
            model_name='storyimage',
            name='modified_by',
            field=models.ForeignKey(related_name='stories_storyimage_modifications', to=settings.AUTH_USER_MODEL, help_text='The user which last modified this item'),
        ),
        migrations.AlterField(
            model_name='storyimage',
            name='story',
            field=models.ForeignKey(related_name='images', to='stories.Story', help_text='The story to associate to'),
        ),
    ]



from __future__ import unicode_literals

"""
Our dashboards typically use HamlPy (https://github.com/nyaruka/django-hamlpy) for templates, but we need our own custom
template loaders because we allow templates to be overridden even when the extension doesn't match, i.e. a template
called index.haml can override index.html in Smartmin
"""

import os

from django.template import TemplateDoesNotExist
from django.template.base import Origin
from django.template.loaders import filesystem, app_directories

from hamlpy import HAML_EXTENSIONS
from hamlpy.compiler import Compiler
from hamlpy.template.utils import get_django_template_loaders


def get_haml_loader(loader):
    if hasattr(loader, 'Loader'):
        baseclass = loader.Loader
    else:
        class baseclass(object):
            def load_template_source(self, *args, **kwargs):
                return loader.load_template_source(*args, **kwargs)

    class Loader(baseclass):
        def load_template_source(self, template_name, *args, **kwargs):
            """
            Used by Django 1.7, 1.8
            """
            _name, _extension = os.path.splitext(template_name)

            for extension in HAML_EXTENSIONS:
                try:
                    haml_source, template_path = super(Loader, self).load_template_source(
                        self._generate_template_name(_name, extension), *args, **kwargs
                    )
                except TemplateDoesNotExist:
                    pass
                else:
                    haml_parser = Compiler()
                    html = haml_parser.process(haml_source)

                    return html, template_path

            raise TemplateDoesNotExist(template_name)

        load_template_source.is_usable = True

        def get_contents(self, origin):
            """
            Used by Django 1.9+
            """
            name, _extension = os.path.splitext(origin.name)
            template_name, _extension = os.path.splitext(origin.template_name)

            for extension in HAML_EXTENSIONS:
                try_name = self._generate_template_name(name, extension)
                try_template_name = self._generate_template_name(template_name, extension)
                try_origin = Origin(try_name, try_template_name, origin.loader)
                try:
                    haml_source = super(Loader, self).get_contents(try_origin)
                except TemplateDoesNotExist:
                    pass
                else:
                    haml_parser = Compiler()
                    return haml_parser.process(haml_source)

            raise TemplateDoesNotExist(origin.template_name)

        def _generate_template_name(self, name, extension="hamlpy"):
            return "%s.%s" % (name, extension)

    return Loader


haml_loaders = dict((name, get_haml_loader(loader)) for (name, loader) in get_django_template_loaders())


HamlFilesystemLoader = get_haml_loader(filesystem)
HamlAppDirectoriesLoader = get_haml_loader(app_directories)



from __future__ import unicode_literals

from dash.orgs.views import OrgObjPermsMixin, OrgPermsMixin
from django.core.urlresolvers import reverse
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _
from smartmin.views import SmartCRUDL, SmartListView, SmartUpdateView, SmartCreateView
from .models import DashBlockType, DashBlock, DashBlockImage


class DashBlockTypeCRUDL(SmartCRUDL):
    model = DashBlockType
    actions = ('create', 'update', 'list')

    class List(SmartListView):
        title = _("Content Types")
        fields = ('name', 'slug', 'description')
        link_fields = ('name',)


class DashBlockFormMixin(object):

    def get_type(self):
        block_type = self.request.POST.get('type', self.request.GET.get('type', None))
        if block_type:
            return DashBlockType.objects.filter(id=block_type).first()
        return None

    def get_context_data(self, *args, **kwargs):
        context = super(DashBlockFormMixin, self).get_context_data(*args, **kwargs)
        context['type'] = self.get_type()
        return context

    def get_success_url(self):
        url = reverse('dashblocks.dashblock_list')
        return "%s?type=%d" % (url, self.object.dashblock_type.id)

    def derive_exclude(self):
        exclude = super(DashBlockFormMixin, self).derive_exclude()

        block_type = self.get_type()
        if block_type:
            if not self.request.user.has_perm(self.permission):
                exclude.append('dashblock_type')

            if not block_type.has_summary:
                exclude.append('summary')

            if not block_type.has_video:
                exclude.append('video_id')

            if not block_type.has_title:
                exclude.append('title')

            if not block_type.has_tags:
                exclude.append('tags')

            if not block_type.has_image:
                exclude.append('image')

            if not block_type.has_link:
                exclude.append('link')

            if not block_type.has_color:
                exclude.append('color')

        # if this user does not have global permissins, remove org as a field
        if not self.request.user.has_perm(self.permission):
            exclude.append('org')

        return exclude

    def pre_save(self, obj):
        obj = super(DashBlockFormMixin, self).pre_save(obj)

        block_type = self.get_type()
        if block_type:
            obj.dashblock_type = block_type

        # if the user doesn't have global permissions, set the org appropriately
        if not self.request.user.has_perm(self.permission):
            obj.org = self.request.org

        obj.space_tags()
        return obj


class DashBlockCRUDL(SmartCRUDL):
    model = DashBlock
    permissions = True
    actions = ('create', 'update', 'list')

    class List(OrgPermsMixin, SmartListView):
        fields = ('title', 'priority', 'dashblock_type', 'tags')
        link_fields = ('title',)
        default_order = '-modified_on'
        search_fields = (
            'title__icontains', 'content__icontains', 'summary__icontains')

        def derive_fields(self):
            fields = super(DashBlockCRUDL.List, self).derive_fields()
            block_type = self.get_type()
            if block_type:
                if not block_type.has_tags:
                    fields = type(fields)(x for x in fields if x != 'tags')
            return fields

        def get_title(self, obj):
            block_type = self.get_type()
            if block_type:
                if not block_type.has_title:
                    return force_text(obj)
            return obj.title

        def derive_title(self):
            type = self.get_type()
            if not type:
                return _("Content Blocks")
            else:
                return _("%s Blocks") % type.name

        def get_type(self):
            block_type = self.request.POST.get('type', self.request.GET.get('type', None))
            if block_type and block_type != '0':
                return DashBlockType.objects.filter(id=block_type).first()
            slug = self.request.POST.get('slug', self.request.GET.get('slug', None))
            if slug:
                return DashBlockType.objects.filter(slug=slug).first()
            return None

        def get_queryset(self, **kwargs):
            queryset = super(DashBlockCRUDL.List, self).get_queryset(**kwargs)

            dashblock_type = self.get_type()
            if dashblock_type:
                queryset = queryset.filter(dashblock_type=dashblock_type)

            queryset = queryset.filter(dashblock_type__is_active=True)
            queryset = queryset.filter(org=self.request.org)

            return queryset

        def get_context_data(self, *args, **kwargs):
            context = super(DashBlockCRUDL.List, self).get_context_data(*args, **kwargs)
            context['types'] = DashBlockType.objects.filter(is_active=True)
            context['filtered_type'] = self.get_type()
            return context

    class Update(OrgObjPermsMixin, DashBlockFormMixin, SmartUpdateView):
        fields = (
            'title', 'summary', 'content', 'image', 'color', 'link',
            'video_id', 'tags', 'dashblock_type', 'priority', 'is_active')

        def get_type(self):
            return self.object.dashblock_type

        def derive_title(self):
            return _("Edit %s") % self.get_type().name

    class Create(OrgPermsMixin, DashBlockFormMixin, SmartCreateView):
        grant_permissions = ('dashblocks.change_dashblock',)

        def derive_initial(self, *args, **kwargs):
            initial = super(DashBlockCRUDL.Create, self).derive_initial(*args, **kwargs)
            dashblock_type = self.get_type()
            other_blocks = DashBlock.objects.filter(
                is_active=True,
                org=self.derive_org(),
                dashblock_type=dashblock_type,
            )
            other_blocks = other_blocks.order_by('-priority')
            if not other_blocks:
                initial['priority'] = 0
            else:
                initial['priority'] = other_blocks[0].priority + 1

            return initial

        def derive_title(self):
            block_type = self.get_type()
            if block_type:
                return _("Create %s") % block_type.name
            else:
                return _("Create Content Block")


class DashBlockImageCRUDL(SmartCRUDL):
    model = DashBlockImage
    actions = ('create', 'update', 'list')

    class Update(SmartUpdateView):
        exclude = (
            'dashblock', 'modified_by', 'modified_on', 'created_on',
            'created_by', 'width', 'height')
        title = "Edit Image"
        success_message = "Image edited successfully."

        def get_success_url(self):
            return reverse('dashblocks.dashblock_update', args=[self.object.dashblock.id])

    class Create(SmartCreateView):
        exclude = (
            'dashblock', 'is_active', 'modified_by', 'modified_on',
            'created_on', 'created_by', 'width', 'height')
        title = "Add Image"
        success_message = "Image added successfully."

        def derive_initial(self, *args, **kwargs):
            initial = super(DashBlockImageCRUDL.Create, self).derive_initial(*args, **kwargs)
            block_id = self.request.POST.get('dashblock', self.request.GET.get('dashblock', None))
            dashblock = DashBlock.objects.get(pk=block_id)
            images = dashblock.sorted_images()
            if not images:
                initial['priority'] = 0
            else:
                initial['priority'] = images[0].priority + 1
            return initial

        def get_success_url(self):
            return reverse('dashblocks.dashblock_update', args=[self.object.dashblock.id])

        def pre_save(self, obj):
            obj = super(DashBlockImageCRUDL.Create, self).pre_save(obj)
            block_id = self.request.POST.get('dashblock', self.request.GET.get('dashblock', None))
            obj.dashblock = DashBlock.objects.filter(pk=block_id).first()
            return obj



from __future__ import unicode_literals

from . import views


urlpatterns = views.OrgCRUDL().as_urlpatterns()
urlpatterns += views.OrgBackgroundCRUDL().as_urlpatterns()
urlpatterns += views.TaskCRUDL().as_urlpatterns()



from __future__ import unicode_literals, print_function

import os

from celery import Celery

os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'dash_test_runner.settings')

from django.conf import settings  # noqa


app = Celery('dash_test_runner')
app.config_from_object('django.conf:settings')
app.autodiscover_tasks(lambda: settings.INSTALLED_APPS)


@app.task(bind=True)
def debug_task(self):
    print('Request: {0!r}'.format(self.request))



from __future__ import unicode_literals

from django import forms
from django.conf import settings
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from timezone_field import TimeZoneFormField
from .models import Org


class CreateOrgLoginForm(forms.Form):
    first_name = forms.CharField(help_text=_("Your first name"))
    last_name = forms.CharField(help_text=_("Your last name"))
    email = forms.EmailField(help_text=_("Your email address"))
    password = forms.CharField(
        widget=forms.PasswordInput,
        help_text=_("Your password, at least eight letters please"))

    def clean_email(self):
        email = self.cleaned_data['email']
        if email:
            if User.objects.filter(username__iexact=email):
                raise forms.ValidationError(_("That email address is already used"))
        return email.lower()

    def clean_password(self):
        password = self.cleaned_data['password']
        if password:
            if not len(password) >= 8:
                raise forms.ValidationError(_("Passwords must contain at least 8 letters."))
        return password


class OrgForm(forms.ModelForm):
    language = forms.ChoiceField(
        required=False, choices=[('', '')] + list(settings.LANGUAGES))
    timezone = TimeZoneFormField()

    def __init__(self, *args, **kwargs):
        super(OrgForm, self).__init__(*args, **kwargs)
        if 'administrators' in self.fields:
            administrators = User.objects.exclude(username__in=['root', 'root2'])
            administrators = administrators.exclude(pk__lt=0)
            self.fields['administrators'].queryset = administrators

    def clean_domain(self):
        domain = self.cleaned_data['domain'].strip().lower() or None
        if domain and domain == getattr(settings, 'HOSTNAME', ""):
            raise forms.ValidationError(_("This domain is used for subdomains"))
        return domain

    class Meta:
        fields = forms.ALL_FIELDS
        model = Org



from __future__ import unicode_literals

from django.conf import settings
from django.conf.urls import url
from django.contrib.auth.views import logout
from smartmin.users.views import login
from . import views


logout_url = getattr(settings, 'LOGOUT_REDIRECT_URL', None)

urlpatterns = [
    url(r'^login/$',
        login,
        dict(template_name='smartmin/users/login.html'),
        name="users.user_login"),
    url(r'^logout/$',
        logout,
        dict(redirect_field_name='go', next_page=logout_url),
        name="users.user_logout"),
]

urlpatterns += views.UserCRUDL().as_urlpatterns()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('categories', '0002_auto_20140820_1415'),
    ]

    operations = [
        migrations.CreateModel(
            name='CategoryImage',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('is_active', models.BooleanField(default=True, help_text='Whether this item is active, use this instead of deleting')),
                ('created_on', models.DateTimeField(help_text='When this item was originally created', auto_now_add=True)),
                ('modified_on', models.DateTimeField(help_text='When this item was last modified', auto_now=True)),
                ('name', models.CharField(help_text='The name to describe this image', max_length=64)),
                ('image', models.ImageField(help_text='The image file to use', upload_to='categories')),
                ('category', models.ForeignKey(help_text='The category this image represents', to='categories.Category')),
                ('created_by', models.ForeignKey(help_text='The user which originally created this item', to=settings.AUTH_USER_MODEL)),
                ('modified_by', models.ForeignKey(help_text='The user which last modified this item', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('stories', '0010_auto_20140922_1514'),
    ]

    operations = [
        migrations.AddField(
            model_name='story',
            name='audio_link',
            field=models.URLField(help_text='A link to an mp3 file to publish on this story', max_length=255, null=True, blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations

def populate_category_images(apps, schema_editor):
    Category = apps.get_model('categories', "Category")
    CategoryImage = apps.get_model('categories', "CategoryImage")
    User = apps.get_model("auth", "User")
    root = User.objects.filter(username="root").first()

    if not root:
        root = User.objects.filter(username="root2").first()

    if not root:
        root = User.objects.create(username="root2")

    for category in Category.objects.all():
        CategoryImage.objects.create(name=category.name,
                                     category=category,
                                     image=category.image,
                                     created_by=root,
                                     modified_by=root)


class Migration(migrations.Migration):

    dependencies = [
        ('categories', '0003_categoryimage'),
    ]

    operations = [
        migrations.RunPython(populate_category_images),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('categories', '0004_auto_20140904_0927'),
    ]

    operations = [
        migrations.AlterField(
            model_name='category',
            name='created_by',
            field=models.ForeignKey(related_name='categories_category_creations', to=settings.AUTH_USER_MODEL, help_text='The user which originally created this item'),
        ),
        migrations.AlterField(
            model_name='category',
            name='modified_by',
            field=models.ForeignKey(related_name='categories_category_modifications', to=settings.AUTH_USER_MODEL, help_text='The user which last modified this item'),
        ),
        migrations.AlterField(
            model_name='categoryimage',
            name='category',
            field=models.ForeignKey(related_name='images', to='categories.Category', help_text='The category this image represents'),
        ),
        migrations.AlterField(
            model_name='categoryimage',
            name='created_by',
            field=models.ForeignKey(related_name='categories_categoryimage_creations', to=settings.AUTH_USER_MODEL, help_text='The user which originally created this item'),
        ),
        migrations.AlterField(
            model_name='categoryimage',
            name='modified_by',
            field=models.ForeignKey(related_name='categories_categoryimage_modifications', to=settings.AUTH_USER_MODEL, help_text='The user which last modified this item'),
        ),
    ]



from __future__ import unicode_literals

from . import views


urlpatterns = views.DashBlockTypeCRUDL().as_urlpatterns()
urlpatterns += views.DashBlockCRUDL().as_urlpatterns()
urlpatterns += views.DashBlockImageCRUDL().as_urlpatterns()



from __future__ import unicode_literals

"""
Django settings for dash_test_runner project.

For more information on this file, see
https://docs.djangoproject.com/en/dev/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/dev/ref/settings/
"""

import logging
import os
import warnings

from django.utils.translation import ugettext_lazy as _


# Disable logging during test runs.
logging.disable(logging.CRITICAL)


# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(__file__))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/dev/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '*z(s8sb^&p-6n#b!w=1-d2bho*+*0g_51bnx=-@a$wj6dnpd2w'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

TEMPLATE_DEBUG = DEBUG

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    'guardian',
    'timezone_field',

    # compress our CSS and js
    'compressor',

    # thumbnail
    'sorl.thumbnail',

    'smartmin',
    'smartmin.users',

    # Uncomment the next line to enable the admin:
    'django.contrib.admin',
    # Uncomment the next line to enable admin documentation:
    # 'django.contrib.admindocs',

    # dash
    'dash.orgs',
    'dash.categories',
    'dash.dashblocks',
    'dash.stories',
    'dash.utils',

    'dash_test_runner.testapp'
)

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'smartmin.users.middleware.ChangePasswordMiddleware',
    'smartmin.middleware.TimezoneMiddleware',
    'dash.orgs.middleware.SetOrgMiddleware',
)

warnings.filterwarnings('error', r"DateTimeField received a naive datetime",
                        RuntimeWarning, r'django\.db\.models\.fields')

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.contrib.auth.context_processors.auth',
    'django.core.context_processors.debug',
    'django.core.context_processors.i18n',
    'django.core.context_processors.media',
    'django.core.context_processors.static',
    'django.contrib.messages.context_processors.messages',
    'django.core.context_processors.request',
    'dash.orgs.context_processors.user_group_perms_processor',
    'dash.orgs.context_processors.set_org_processor',
)


ROOT_URLCONF = 'dash_test_runner.urls'

WSGI_APPLICATION = 'dash_test_runner.wsgi.application'


# Database
# https://docs.djangoproject.com/en/dev/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}


SITE_ID = 1

# Internationalization
# https://docs.djangoproject.com/en/dev/topics/i18n/

LANGUAGE_CODE = 'en-us'


# Available languages for translation
LANGUAGES = (('en', "English"), ('fr', "French"))
DEFAULT_LANGUAGE = "en"


TIME_ZONE = 'UTC'
USER_TIME_ZONE = 'Africa/Kigali'


USE_I18N = True

USE_L10N = True

USE_TZ = True

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/home/media/media.lawrence.com/static/"
STATIC_ROOT = ''

# URL prefix for static files.
# Example: "http://media.lawrence.com/static/"
STATIC_URL = '/static/'

# URL prefix for admin static files -- CSS, JavaScript and images.
# Make sure to use a trailing slash.
# Examples: "http://foo.com/static/admin/", "/static/admin/".
ADMIN_MEDIA_PREFIX = '/static/admin/'

# Additional locations of static files
STATICFILES_DIRS = (
    # Put strings here, like "/home/html/static" or "C:/www/django/static".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    # 'django.contrib.staticfiles.finders.DefaultStorageFinder',
)

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'dash.utils.haml.HamlFilesystemLoader',
    'dash.utils.haml.HamlAppDirectoriesLoader',
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

AUTHENTICATION_BACKENDS = (
    'django.contrib.auth.backends.ModelBackend',  # this is default
    'guardian.backends.ObjectPermissionBackend',
)

# create the smartmin CRUDL permissions on all objects
PERMISSIONS = {
    '*': ('create',  # can create an object
          'read',   # can read an object, viewing it's details
          'update',  # can update an object
          'delete',  # can delete an object,
          'list'),  # can view a list of the objects
    'auth.user': ('profile', 'forget', 'recover', 'expired', 'failed', 'newpassword', 'mimic'),
    'orgs.org': ('choose', 'home', 'edit', 'manage_accounts', 'create_login', 'join'),
    'stories.story': ('images',),
}

# assigns the permissions that each group should have, here creating an Administrator group with
# authority to create and change users
GROUP_PERMISSIONS = {
    "Administrators": (
        'categories.category.*',
        'categories.categoryimage.*',
        'dashblocks.dashblock.*',
        'users.user_profile',
        'orgs.org_home',
        'orgs.org_edit',
        'orgs.org_manage_accounts',
        'orgs.orgbackground.*',
        'stories.story.*',

    ),
    "Editors": (
        'categories.category.*',
        'categories.categoryimage.*',
        'dashblocks.dashblock.*',
        'dashblocks.dashblocktype.*',
        'news.newsitem.*',
        'news.video.*',
        'orgs.org_home',
        'orgs.org_profile',
        'polls.poll.*',
        'polls.pollcategory.*',
        'polls.pollimage.*',
        'polls.featuredresponse.*',
        'stories.story.*',
        'stories.storyimage.*',
        'users.user_profile',
    ),
    "Viewers": [],
}

ANONYMOUS_USER_ID = -1
LOGIN_URL = '/users/login/'
LOGIN_REDIRECT_URL = "/manage/org/choose/"
LOGOUT_REDIRECT_URL = "/"

# ----------------------------------------------------------------------------
# Async tasks with django-celery
# ----------------------------------------------------------------------------

CELERY_RESULT_BACKEND = None

BROKER_BACKEND = 'redis'
BROKER_HOST = 'localhost'
BROKER_PORT = 6379
BROKER_VHOST = '4'

# RapidPRO
SITE_API_HOST = 'http://localhost:8001'
HOSTNAME = 'ureport.io'
SITE_CHOOSER_TEMPLATE = 'orgs/org_chooser.html'
SITE_CHOOSER_URL_NAME = 'orgs.org_chooser'
SITE_ALLOW_NO_ORG = ('orgs.task_list',)

CACHES = {
    'default': {
        'BACKEND': 'django_redis.cache.RedisCache',
        'LOCATION': 'redis://127.0.0.1:6379/10',
        'OPTIONS': {
            'CLIENT_CLASS': 'django_redis.client.DefaultClient',
        }
    }
}

ORG_CONFIG_FIELDS = [
    dict(name='shortcode',
         field=dict(help_text=_("The shortcode that users will use to contact U-report locally"),
                    required=True)),
    dict(name='join_text',
         field=dict(help_text=_("The short text used to direct visitors to join U-report"),
                    required=False)),
    dict(name='join_fg_color',
         field=dict(help_text=_("The color used to draw the text on the join bar"),
                    required=False),
         superuser_only=True),
    dict(name='join_bg_color',
         field=dict(help_text=_("The color used to draw the background on the join bar"),
                    required=False),
         superuser_only=True),
    dict(name='primary_color',
         field=dict(help_text=_("The primary color for styling for this organization"),
                    required=False),
         superuser_only=True),
    dict(name='secondary_color',
         field=dict(help_text=_("The secondary color for styling for this organization"),
                    required=False),
         superuser_only=True),
    dict(name='bg_color',
         field=dict(help_text=_("The background color for the site"),
                    required=False),
         superuser_only=True),
    dict(name='colors',
         field=dict(help_text=_("Up to 6 colors for styling charts, use comma between colors"),
                    required=False),
         superuser_only=True),
    dict(name='google_tracking_id',
         field=dict(help_text=_("The Google Analytics Tracking ID for this organization"),
                    required=False)),
    dict(name='youtube_channel_url',
         field=dict(help_text=_("The URL to the Youtube channel for this organization"),
                    required=False)),
    dict(name='facebook_page_url',
         field=dict(help_text=_("The URL to the Facebook page for this organization"),
                    required=False)),
    dict(name='twitter_handle',
         field=dict(help_text=_("The Twitter handle for this organization"),
                    required=False)),
    dict(name='twitter_user_widget',
         field=dict(help_text=_("The Twitter widget used for following new users"),
                    required=False)),
    dict(name='twitter_search_widget',
         field=dict(help_text=_("The Twitter widget used for searching"),
                    required=False)),
    dict(name='reporter_group',
         field=dict(help_text=_("The name of txbhe Contact Group that contains "
                                "registered reporters")),
         superuser_only=True),
    dict(name='born_label',
         field=dict(help_text=_("The label of the Contact Field that contains "
                                "the birth date of reporters")),
         superuser_only=True),
    dict(name='gender_label',
         field=dict(help_text=_("The label of the Contact Field that contains "
                                "the gender of reporters")),
         superuser_only=True),
    dict(name='occupation_label',
         field=dict(help_text=_("The label of the Contact Field that contains "
                                "the occupation of reporters")),
         superuser_only=True),
    dict(name='registration_label',
         field=dict(help_text=_("The label of the Contact Field that contains "
                                "the registration date of reporters")),
         superuser_only=True),
    dict(name='state_label',
         field=dict(help_text=_("The label of the Contact Field that contains "
                                "the State of reporters")),
         superuser_only=True),
    dict(name='district_label',
         field=dict(help_text=_("The label of the Contact Field that contains "
                                "the District of reporters")),
         superuser_only=True),
    dict(name='male_label',
         field=dict(help_text=_("The label assigned to U-reporters that are Male.")),
         superuser_only=True),
    dict(name='female_label',
         field=dict(help_text=_("The label assigned to U-reporters that are Female.")),
         superuser_only=True),
    dict(name='has_jobs',
         field=dict(help_text=_("If there are jobs to be shown on the public site"),
                    required=False)),
    # dict(name='featured_state',
    #      field=dict(help_text=_("Choose the featured State of reporters "
    #                             "shown on the home page"))),
]

# ----------------------------------------------------------------------------
# Directory Configuration
# ----------------------------------------------------------------------------

PROJECT_DIR = os.path.join(os.path.abspath(os.path.dirname(__file__)))
RESOURCES_DIR = os.path.join(PROJECT_DIR, '../resources')
TESTFILES_DIR = os.path.join(PROJECT_DIR, '../testfiles')
MEDIA_ROOT = os.path.join(PROJECT_DIR, '../media')
MEDIA_URL = "/media/"

TEMPLATE_DIRS = [os.path.join(PROJECT_DIR, '../templates')]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations

def generate_initial_block_types(apps, schema_editor):
    User = apps.get_model("auth", "User")
    root = User.objects.filter(username="root").first()

    if not root:
        root = User.objects.filter(username='root2').first()

        if not root:
            root = User.objects.create(username="root2")

    DashBlockType = apps.get_model("dashblocks", "DashBlockType")

    DashBlockType.objects.get_or_create(name="About",
                                        slug="about",
                                        description="About pages by highest priority",
                                        has_title=True,
                                        has_image=True,
                                        has_rich_text=False,
                                        has_summary=False,
                                        has_link=False,
                                        has_gallery=False,
                                        has_color=False,
                                        has_video=False,
                                        has_tags=False,
                                        created_by=root,
                                        modified_by=root)

    DashBlockType.objects.get_or_create(name="Join & Engage",
                                        slug="join_engage",
                                        description="Join & Engage by highest priority",
                                        has_title=True,
                                        has_image=True,
                                        has_rich_text=False,
                                        has_summary=False,
                                        has_link=False,
                                        has_gallery=False,
                                        has_color=False,
                                        has_video=False,
                                        has_tags=False,
                                        created_by=root,
                                        modified_by=root)


    DashBlockType.objects.get_or_create(name="YouTube Videos",
                                        slug="youtube_videos",
                                        description="YouTube Videos to embed on the website",
                                        has_title=True,
                                        has_image=False,
                                        has_rich_text=False,
                                        has_summary=False,
                                        has_link=False,
                                        has_gallery=False,
                                        has_color=False,
                                        has_video=True,
                                        has_tags=False,
                                        created_by=root,
                                        modified_by=root)



class Migration(migrations.Migration):

    dependencies = [
        ('dashblocks', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(generate_initial_block_types),
    ]



from __future__ import unicode_literals

import json
import random

from dash.dash_email import send_dash_email
from dash.utils import datetime_to_ms
from datetime import datetime
from django.conf import settings
from django.contrib.auth.models import User, Group
from django.core.cache import cache
from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import force_text, python_2_unicode_compatible
from smartmin.models import SmartModel
from temba_client.v1 import TembaClient as TembaClient1
from temba_client.v2 import TembaClient as TembaClient2
from timezone_field import TimeZoneField


STATE = 1
DISTRICT = 2

# we cache boundary data for a month at a time
BOUNDARY_CACHE_TIME = getattr(settings, 'API_BOUNDARY_CACHE_TIME', 60 * 60 * 24 * 30)

BOUNDARY_CACHE_KEY = 'org:%d:boundaries'
BOUNDARY_LEVEL_1_KEY = 'geojson:%d'
BOUNDARY_LEVEL_2_KEY = 'geojson:%d:%s'


@python_2_unicode_compatible
class Org(SmartModel):
    name = models.CharField(
        verbose_name=_("Name"), max_length=128,
        help_text=_("The name of this organization"))

    logo = models.ImageField(
        upload_to='logos', null=True, blank=True,
        help_text=_("The logo that should be used for this organization"))

    administrators = models.ManyToManyField(
        User, verbose_name=_("Administrators"), related_name="org_admins",
        help_text=_("The administrators in your organization"))

    viewers = models.ManyToManyField(
        User, verbose_name=_("Viewers"), related_name="org_viewers",
        help_text=_("The viewers in your organization"))

    editors = models.ManyToManyField(
        User, verbose_name=_("Editors"), related_name="org_editors",
        help_text=_("The editors in your organization"))

    language = models.CharField(
        verbose_name=_("Language"), max_length=64, null=True, blank=True,
        help_text=_("The main language used by this organization"))

    subdomain = models.SlugField(
        verbose_name=_("Subdomain"), null=True, blank=True, max_length=255, unique=True,
        error_messages=dict(unique=_("This subdomain is not available")),
        help_text=_("The subdomain for this organization"))

    domain = models.CharField(
        verbose_name=_("Domain"), null=True, blank=True, max_length=255, unique=True,
        error_messages=dict(unique=_("This domain is not available")),
        help_text=_("The custom domain for this organization"))

    timezone = TimeZoneField(
        verbose_name=_("Timezone"), default='UTC',
        help_text=_("The timezone your organization is in."))

    api_token = models.CharField(
        max_length=128, null=True, blank=True,
        help_text=_("The API token for the RapidPro account this dashboard "
                    "is tied to"))

    config = models.TextField(
        null=True, blank=True,
        help_text=_("JSON blob used to store configuration information "
                    "associated with this organization"))

    def get_config(self, name, default=None):
        config = getattr(self, '_config', None)

        if config is None:
            if not self.config:
                return default

            config = json.loads(self.config)
            self._config = config

        return config.get(name, default)

    def set_config(self, name, value, commit=True):
        if not self.config:
            config = dict()
        else:
            config = json.loads(self.config)

        config[name] = value
        self.config = json.dumps(config)
        self._config = config

        if commit:
            self.save()

    def get_org_admins(self):
        return self.administrators.all()

    def get_org_editors(self):
        return self.editors.all()

    def get_org_viewers(self):
        return self.viewers.all()

    def get_org_users(self):
        org_users = self.get_org_admins() | self.get_org_editors() | self.get_org_viewers()
        return org_users.distinct()

    def get_user_org_group(self, user):
        if user in self.get_org_admins():
            user._org_group = Group.objects.get(name="Administrators")
        elif user in self.get_org_editors():
            user._org_group = Group.objects.get(name="Editors")
        elif user in self.get_org_viewers():
            user._org_group = Group.objects.get(name="Viewers")
        else:
            user._org_group = None

        return getattr(user, '_org_group', None)

    def get_user(self):
        user = self.administrators.filter(is_active=True).first()
        if user:
            org_user = user
            org_user.set_org(self)
            return org_user

    def get_temba_client(self, api_version=1):
        if api_version not in (1, 2):
            raise ValueError("Unsupported RapidPro API version: %d" % api_version)

        host = getattr(settings, 'SITE_API_HOST', None)
        agent = getattr(settings, 'SITE_API_USER_AGENT', None)

        if host.endswith('api/v1') or host.endswith('api/v1/'):
            raise ValueError("API host should not include API version, "
                             "e.g. http://example.com instead of http://example.com/api/v1")

        client_cls = TembaClient1 if api_version == 1 else TembaClient2

        return client_cls(host, self.api_token, user_agent=agent)

    def build_host_link(self, user_authenticated=False):
        host_tld = getattr(settings, "HOSTNAME", 'locahost')
        is_secure = getattr(settings, 'SESSION_COOKIE_SECURE', False)

        prefix = 'http://'

        if self.domain and is_secure and not user_authenticated:
            return prefix + str(self.domain)

        if is_secure:
            prefix = 'https://'

        if self.subdomain == '':
            return prefix + host_tld
        return prefix + force_text(self.subdomain) + "." + host_tld

    @classmethod
    def rebuild_org_boundaries_task(cls, org):
        from dash.orgs.tasks import rebuild_org_boundaries
        rebuild_org_boundaries.delay(org.pk)

    def build_boundaries(self):

        this_time = datetime.now()
        temba_client = self.get_temba_client()
        client_boundaries = temba_client.get_boundaries()

        # we now build our cached versions of level 1 (all states) and level 2
        # (all districts for each state) geojson
        states = []
        districts_by_state = dict()
        for boundary in client_boundaries:
            if boundary.level == STATE:
                states.append(boundary)
            elif boundary.level == DISTRICT:
                osm_id = boundary.parent
                if osm_id not in districts_by_state:
                    districts_by_state[osm_id] = []

                districts = districts_by_state[osm_id]
                districts.append(boundary)

        # mini function to convert a list of boundary objects to geojson
        def to_geojson(boundary_list):
            features = [dict(type='Feature',
                             geometry=dict(type=b.geometry.type,
                                           coordinates=b.geometry.coordinates),
                             properties=dict(name=b.name, id=b.boundary, level=b.level))
                        for b in boundary_list]
            return dict(type='FeatureCollection', features=features)

        boundaries = dict()
        boundaries[BOUNDARY_LEVEL_1_KEY % self.id] = to_geojson(states)

        for state_id in districts_by_state.keys():
            boundaries[BOUNDARY_LEVEL_2_KEY % (self.id, state_id)] = to_geojson(
                districts_by_state[state_id])

        key = BOUNDARY_CACHE_KEY % self.pk
        value = {'time': datetime_to_ms(this_time), 'results': boundaries}
        cache.set(key, value, BOUNDARY_CACHE_TIME)

        return boundaries

    def get_boundaries(self):
        key = BOUNDARY_CACHE_KEY % self.pk
        cached_value = cache.get(key, None)
        if cached_value:
            return cached_value['results']
        Org.rebuild_org_boundaries_task(self)

    def get_country_geojson(self):
        boundaries = self.get_boundaries()
        if boundaries:
            key = BOUNDARY_LEVEL_1_KEY % self.id
            return boundaries.get(key, None)

    def get_state_geojson(self, state_id):
        boundaries = self.get_boundaries()
        if boundaries:
            key = BOUNDARY_LEVEL_2_KEY % (self.id, state_id)
            return boundaries.get(key, None)

    def get_top_level_geojson_ids(self):
        org_country_boundaries = self.get_country_geojson()
        return [elt['properties']['id'] for elt in org_country_boundaries['features']]

    def get_task_state(self, task_key):
        return TaskState.get_or_create(self, task_key)

    @classmethod
    def create_user(cls, email, password):
        user = User.objects.create_user(username=email, email=email, password=password)
        return user

    @classmethod
    def get_org(cls, user):
        if not user:
            return None

        if not hasattr(user, '_org'):
            org = Org.objects.filter(administrators=user, is_active=True).first()
            if org:
                user._org = org

        return getattr(user, '_org', None)

    def __str__(self):
        return self.name


def get_org(obj):
    return getattr(obj, '_org', None)


def set_org(obj, org):
    obj._org = org


def get_user_orgs(user):
    if user.is_superuser:
        return Org.objects.all()
    user_orgs = user.org_admins.all() | user.org_editors.all() | user.org_viewers.all()
    return user_orgs.distinct()


def get_org_group(obj):
    org_group = None
    org = obj.get_org()
    if org:
        org_group = org.get_user_org_group(obj)
    return org_group


User.get_org = get_org
User.set_org = set_org
User.get_user_orgs = get_user_orgs
User.get_org_group = get_org_group


USER_GROUPS = (('A', _("Administrator")),
               ('E', _("Editor")),
               ('V', _("Viewer")))


class Invitation(SmartModel):
    org = models.ForeignKey(
        Org, verbose_name=_("Org"), related_name="invitations",
        help_text=_("The organization to which the account is invited to view"))

    email = models.EmailField(
        verbose_name=_("Email"),
        help_text=_("The email to which we send the invitation of the viewer"))

    secret = models.CharField(
        verbose_name=_("Secret"), max_length=64, unique=True,
        help_text=_("a unique code associated with this invitation"))

    user_group = models.CharField(
        max_length=1, choices=USER_GROUPS, default='V', verbose_name=_("User Role"))

    def save(self, *args, **kwargs):
        if not self.secret:
            secret = Invitation.generate_random_string(64)

            while Invitation.objects.filter(secret=secret):
                secret = Invitation.generate_random_string(64)

            self.secret = secret

        return super(Invitation, self).save(*args, **kwargs)

    @classmethod
    def generate_random_string(cls, length):
        """
        Generatesa a [length] characters alpha numeric secret
        """
        # avoid things that could be mistaken ex: 'I' and '1'
        letters = "23456789ABCDEFGHJKLMNPQRSTUVWXYZ"
        return ''.join([random.choice(letters) for _ in range(length)])

    def send_invitation(self):
        from .tasks import send_invitation_email_task
        send_invitation_email_task(self.id)

    def send_email(self):
        # no=op if we do not know the email
        if not self.email:
            return

        subject = _("%s Invitation") % self.org.name
        template = "orgs/email/invitation_email"
        to_email = self.email

        context = dict(org=self.org, now=timezone.now(), invitation=self)
        context['subject'] = subject
        context['host'] = self.org.build_host_link()

        send_dash_email(to_email, subject, template, context)


class OrgBackground(SmartModel):
    BACKGROUND_TYPES = (('B', _("Banner")),
                        ('P', _("Pattern")))

    org = models.ForeignKey(
        Org, verbose_name=_("Org"), related_name="backgrounds",
        help_text=_("The organization in which the image will be used"))

    name = models.CharField(
        verbose_name=_("Name"), max_length=128,
        help_text=_("The name to describe this background"))

    background_type = models.CharField(
        max_length=1, choices=BACKGROUND_TYPES, default='P', verbose_name=_("Background type"))

    image = models.ImageField(upload_to='org_bgs', help_text=_("The image file"))


class TaskState(models.Model):
    """
    Holds org specific state for a scheduled task
    """
    org = models.ForeignKey(Org, related_name='task_states')

    task_key = models.CharField(max_length=32)

    started_on = models.DateTimeField(null=True)

    ended_on = models.DateTimeField(null=True)

    last_successfully_started_on = models.DateTimeField(null=True)

    last_results = models.TextField(null=True)

    is_failing = models.BooleanField(default=False)

    is_disabled = models.BooleanField(default=False)

    @classmethod
    def get_or_create(cls, org, task_key):
        existing = cls.objects.filter(org=org, task_key=task_key).first()
        if existing:
            return existing

        return cls.objects.create(org=org, task_key=task_key)

    @classmethod
    def get_failing(cls):
        return cls.objects.filter(org__is_active=True, is_failing=True)

    def is_running(self):
        return self.started_on and not self.ended_on

    def has_ever_run(self):
        return self.started_on is not None

    def get_last_results(self):
        return json.loads(self.last_results) if self.last_results else None

    def get_time_taken(self):
        until = self.ended_on if self.ended_on else timezone.now()
        return (until - self.started_on).total_seconds()

    class Meta:
        unique_together = ('org', 'task_key')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('stories', '0002_auto_20140805_1158'),
    ]

    operations = [
        migrations.AddField(
            model_name='story',
            name='summary',
            field=models.TextField(help_text='The summary for the story', null=True, blank=True),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import six

from dash.orgs.models import Org
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from smartmin.models import SmartModel


@python_2_unicode_compatible
class Category(SmartModel):
    """
    Every organization can choose to categorize their polls or stories
    according to their needs.
    """
    name = models.CharField(max_length=64,
                            help_text=_("The name of this category"))

    image = models.ImageField(upload_to='categories', null=True, blank=True,
                              help_text=_("An optional image that can describe this category"))

    org = models.ForeignKey(Org, related_name='categories',
                            help_text=_("The organization this category applies to"))

    def get_first_image(self):
        cat_images = self.images.filter(is_active=True).exclude(image='')
        if cat_images and cat_images.first().image:
            return cat_images.first().image

    def get_label_from_instance(self):
        label = str(self)
        if isinstance(label, six.binary_type):
            label = label.decode('utf-8')

        if not self.is_active:
            label = "%s %s" % (label, "(Inactive)")
        return label

    def __str__(self):
        return "%s - %s" % (self.org, self.name)

    class Meta:
        unique_together = ('name', 'org')
        verbose_name_plural = _("Categories")


@python_2_unicode_compatible
class CategoryImage(SmartModel):
    name = models.CharField(max_length=64,
                            help_text=_("The name to describe this image"))

    category = models.ForeignKey(Category, related_name='images',
                                 help_text=_("The category this image represents"))

    image = models.ImageField(upload_to='categories',
                              help_text=_("The image file to use"))

    def __str__(self):
        return "%s - %s" % (self.category.name, self.name)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('orgs', '0003_org_logo'),
    ]

    operations = [
        migrations.AlterField(
            model_name='org',
            name='administrators',
            field=models.ManyToManyField(help_text='The administrators in your organization', to=settings.AUTH_USER_MODEL, verbose_name='Administrators'),
        ),
        migrations.AlterField(
            model_name='org',
            name='editors',
            field=models.ManyToManyField(help_text='The editors in your organization', to=settings.AUTH_USER_MODEL, verbose_name='Editors'),
        ),
        migrations.AlterField(
            model_name='org',
            name='viewers',
            field=models.ManyToManyField(help_text='The viewers in your organization', to=settings.AUTH_USER_MODEL, verbose_name='Viewers'),
        ),
    ]
